# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

WARP_TENSOR_IMAGE_MULTI_TRANSFORM_METADATA = Metadata(
    id="013f15045b6bdcf60fcdbe3fce339b08b8439b03.boutiques",
    name="WarpTensorImageMultiTransform",
    package="ants",
    container_image_tag="antsx/ants:v2.5.3",
)


_WarpTensorImageMultiTransformParamsDictNoTag = typing.TypedDict('_WarpTensorImageMultiTransformParamsDictNoTag', {
    "image_dimension": int,
    "moving_image": InputPathType,
    "output_image": str,
    "reference_image": typing.NotRequired[InputPathType | None],
    "tightest_bounding_box": bool,
    "reslice_by_header": bool,
    "use_nearest_neighbor": bool,
    "transforms": list[str],
    "ants_prefix": typing.NotRequired[str | None],
    "ants_prefix_invert": typing.NotRequired[str | None],
})
WarpTensorImageMultiTransformParamsDictTagged = typing.TypedDict('WarpTensorImageMultiTransformParamsDictTagged', {
    "@type": typing.Literal["ants/WarpTensorImageMultiTransform"],
    "image_dimension": int,
    "moving_image": InputPathType,
    "output_image": str,
    "reference_image": typing.NotRequired[InputPathType | None],
    "tightest_bounding_box": bool,
    "reslice_by_header": bool,
    "use_nearest_neighbor": bool,
    "transforms": list[str],
    "ants_prefix": typing.NotRequired[str | None],
    "ants_prefix_invert": typing.NotRequired[str | None],
})
WarpTensorImageMultiTransformParamsDict = _WarpTensorImageMultiTransformParamsDictNoTag | WarpTensorImageMultiTransformParamsDictTagged


class WarpTensorImageMultiTransformOutputs(typing.NamedTuple):
    """
    Output object returned when calling `WarpTensorImageMultiTransformParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_image_file: OutputPathType
    """The resultant transformed output image."""


def warp_tensor_image_multi_transform_params(
    image_dimension: int,
    moving_image: InputPathType,
    output_image: str,
    transforms: list[str],
    reference_image: InputPathType | None = None,
    tightest_bounding_box: bool = False,
    reslice_by_header: bool = False,
    use_nearest_neighbor: bool = False,
    ants_prefix: str | None = None,
    ants_prefix_invert: str | None = None,
) -> WarpTensorImageMultiTransformParamsDictTagged:
    """
    Build parameters.
    
    Args:
        image_dimension: Dimensionality of the image (e.g., 2D or 3D).
        moving_image: The moving image that will be transformed.
        output_image: Path for saving the transformed output image.
        transforms: List of transformations to apply, which can include\
            deformation fields or affine transforms.
        reference_image: Reference image for reslicing or defining the\
            transformation domain.
        tightest_bounding_box: Compute the tightest bounding box using all\
            affine transformations.
        reslice_by_header: Use the orientation matrix and origin encoded in the\
            image file header for reslicing.
        use_nearest_neighbor: Use Nearest Neighbor Interpolator for the\
            transformation.
        ants_prefix: Prefix for ANTS-generated deformation and affine\
            transformation files.
        ants_prefix_invert: Prefix for inverting ANTS-generated affine and\
            deformation transformations.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "ants/WarpTensorImageMultiTransform",
        "image_dimension": image_dimension,
        "moving_image": moving_image,
        "output_image": output_image,
        "tightest_bounding_box": tightest_bounding_box,
        "reslice_by_header": reslice_by_header,
        "use_nearest_neighbor": use_nearest_neighbor,
        "transforms": transforms,
    }
    if reference_image is not None:
        params["reference_image"] = reference_image
    if ants_prefix is not None:
        params["ants_prefix"] = ants_prefix
    if ants_prefix_invert is not None:
        params["ants_prefix_invert"] = ants_prefix_invert
    return params


def warp_tensor_image_multi_transform_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `WarpTensorImageMultiTransformParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("image_dimension", None) is None:
        raise StyxValidationError("`image_dimension` must not be None")
    if not isinstance(params["image_dimension"], int):
        raise StyxValidationError(f'`image_dimension` has the wrong type: Received `{type(params.get("image_dimension", None))}` expected `int`')
    if params.get("moving_image", None) is None:
        raise StyxValidationError("`moving_image` must not be None")
    if not isinstance(params["moving_image"], (pathlib.Path, str)):
        raise StyxValidationError(f'`moving_image` has the wrong type: Received `{type(params.get("moving_image", None))}` expected `InputPathType`')
    if params.get("output_image", None) is None:
        raise StyxValidationError("`output_image` must not be None")
    if not isinstance(params["output_image"], str):
        raise StyxValidationError(f'`output_image` has the wrong type: Received `{type(params.get("output_image", None))}` expected `str`')
    if params.get("reference_image", None) is not None:
        if not isinstance(params["reference_image"], (pathlib.Path, str)):
            raise StyxValidationError(f'`reference_image` has the wrong type: Received `{type(params.get("reference_image", None))}` expected `InputPathType | None`')
    if params.get("tightest_bounding_box", False) is None:
        raise StyxValidationError("`tightest_bounding_box` must not be None")
    if not isinstance(params["tightest_bounding_box"], bool):
        raise StyxValidationError(f'`tightest_bounding_box` has the wrong type: Received `{type(params.get("tightest_bounding_box", False))}` expected `bool`')
    if params.get("reslice_by_header", False) is None:
        raise StyxValidationError("`reslice_by_header` must not be None")
    if not isinstance(params["reslice_by_header"], bool):
        raise StyxValidationError(f'`reslice_by_header` has the wrong type: Received `{type(params.get("reslice_by_header", False))}` expected `bool`')
    if params.get("use_nearest_neighbor", False) is None:
        raise StyxValidationError("`use_nearest_neighbor` must not be None")
    if not isinstance(params["use_nearest_neighbor"], bool):
        raise StyxValidationError(f'`use_nearest_neighbor` has the wrong type: Received `{type(params.get("use_nearest_neighbor", False))}` expected `bool`')
    if params.get("transforms", None) is None:
        raise StyxValidationError("`transforms` must not be None")
    if not isinstance(params["transforms"], list):
        raise StyxValidationError(f'`transforms` has the wrong type: Received `{type(params.get("transforms", None))}` expected `list[str]`')
    for e in params["transforms"]:
        if not isinstance(e, str):
            raise StyxValidationError(f'`transforms` has the wrong type: Received `{type(params.get("transforms", None))}` expected `list[str]`')
    if params.get("ants_prefix", None) is not None:
        if not isinstance(params["ants_prefix"], str):
            raise StyxValidationError(f'`ants_prefix` has the wrong type: Received `{type(params.get("ants_prefix", None))}` expected `str | None`')
    if params.get("ants_prefix_invert", None) is not None:
        if not isinstance(params["ants_prefix_invert"], str):
            raise StyxValidationError(f'`ants_prefix_invert` has the wrong type: Received `{type(params.get("ants_prefix_invert", None))}` expected `str | None`')


def warp_tensor_image_multi_transform_cargs(
    params: WarpTensorImageMultiTransformParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("WarpTensorImageMultiTransform")
    cargs.append(str(params.get("image_dimension", None)))
    cargs.append(execution.input_file(params.get("moving_image", None)))
    cargs.append(params.get("output_image", None))
    if params.get("reference_image", None) is not None:
        cargs.extend([
            "-R",
            execution.input_file(params.get("reference_image", None))
        ])
    if params.get("tightest_bounding_box", False):
        cargs.append("--tightest-bounding-box")
    if params.get("reslice_by_header", False):
        cargs.append("--reslice-by-header")
    if params.get("use_nearest_neighbor", False):
        cargs.append("--use-NN")
    cargs.extend(params.get("transforms", None))
    if params.get("ants_prefix", None) is not None:
        cargs.extend([
            "--ANTS-prefix",
            params.get("ants_prefix", None)
        ])
    if params.get("ants_prefix_invert", None) is not None:
        cargs.extend([
            "--ANTS-prefix-invert",
            params.get("ants_prefix_invert", None)
        ])
    return cargs


def warp_tensor_image_multi_transform_outputs(
    params: WarpTensorImageMultiTransformParamsDict,
    execution: Execution,
) -> WarpTensorImageMultiTransformOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = WarpTensorImageMultiTransformOutputs(
        root=execution.output_file("."),
        output_image_file=execution.output_file(params.get("output_image", None)),
    )
    return ret


def warp_tensor_image_multi_transform_execute(
    params: WarpTensorImageMultiTransformParamsDict,
    runner: Runner | None = None,
) -> WarpTensorImageMultiTransformOutputs:
    """
    WarpTensorImageMultiTransform
    
    WarpTensorImageMultiTransform is used to apply transformations including
    affine and deformation fields to an image, supporting various interpolation
    techniques, image header reslicing, and compatibility with ANTS-generated
    transformations.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `WarpTensorImageMultiTransformOutputs`).
    """
    warp_tensor_image_multi_transform_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(WARP_TENSOR_IMAGE_MULTI_TRANSFORM_METADATA)
    params = execution.params(params)
    cargs = warp_tensor_image_multi_transform_cargs(params, execution)
    ret = warp_tensor_image_multi_transform_outputs(params, execution)
    execution.run(cargs)
    return ret


def warp_tensor_image_multi_transform(
    image_dimension: int,
    moving_image: InputPathType,
    output_image: str,
    transforms: list[str],
    reference_image: InputPathType | None = None,
    tightest_bounding_box: bool = False,
    reslice_by_header: bool = False,
    use_nearest_neighbor: bool = False,
    ants_prefix: str | None = None,
    ants_prefix_invert: str | None = None,
    runner: Runner | None = None,
) -> WarpTensorImageMultiTransformOutputs:
    """
    WarpTensorImageMultiTransform
    
    WarpTensorImageMultiTransform is used to apply transformations including
    affine and deformation fields to an image, supporting various interpolation
    techniques, image header reslicing, and compatibility with ANTS-generated
    transformations.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        image_dimension: Dimensionality of the image (e.g., 2D or 3D).
        moving_image: The moving image that will be transformed.
        output_image: Path for saving the transformed output image.
        transforms: List of transformations to apply, which can include\
            deformation fields or affine transforms.
        reference_image: Reference image for reslicing or defining the\
            transformation domain.
        tightest_bounding_box: Compute the tightest bounding box using all\
            affine transformations.
        reslice_by_header: Use the orientation matrix and origin encoded in the\
            image file header for reslicing.
        use_nearest_neighbor: Use Nearest Neighbor Interpolator for the\
            transformation.
        ants_prefix: Prefix for ANTS-generated deformation and affine\
            transformation files.
        ants_prefix_invert: Prefix for inverting ANTS-generated affine and\
            deformation transformations.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `WarpTensorImageMultiTransformOutputs`).
    """
    params = warp_tensor_image_multi_transform_params(
        image_dimension=image_dimension,
        moving_image=moving_image,
        output_image=output_image,
        reference_image=reference_image,
        tightest_bounding_box=tightest_bounding_box,
        reslice_by_header=reslice_by_header,
        use_nearest_neighbor=use_nearest_neighbor,
        transforms=transforms,
        ants_prefix=ants_prefix,
        ants_prefix_invert=ants_prefix_invert,
    )
    return warp_tensor_image_multi_transform_execute(params, runner)


__all__ = [
    "WARP_TENSOR_IMAGE_MULTI_TRANSFORM_METADATA",
    "WarpTensorImageMultiTransformOutputs",
    "WarpTensorImageMultiTransformParamsDict",
    "WarpTensorImageMultiTransformParamsDictTagged",
    "warp_tensor_image_multi_transform",
    "warp_tensor_image_multi_transform_execute",
    "warp_tensor_image_multi_transform_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

ANTSUSE_LANDMARK_IMAGES_TO_GET_BSPLINE_DISPLACEMENT_FIELD_METADATA = Metadata(
    id="97a7208c0feb66caac1f85f91d7e5b2be53f6438.boutiques",
    name="ANTSUseLandmarkImagesToGetBSplineDisplacementField",
    package="ants",
    container_image_tag="antsx/ants:v2.5.3",
)


_AntsuseLandmarkImagesToGetBsplineDisplacementFieldParamsDictNoTag = typing.TypedDict('_AntsuseLandmarkImagesToGetBsplineDisplacementFieldParamsDictNoTag', {
    "fixed_image_with_labeled_landmarks": InputPathType,
    "moving_image_with_labeled_landmarks": InputPathType,
    "output_displacement_field": str,
    "mesh_size": str,
    "number_of_levels": int,
    "order": typing.NotRequired[int | None],
    "enforce_stationary_boundaries": typing.NotRequired[int | None],
    "landmark_weights": typing.NotRequired[InputPathType | None],
})
AntsuseLandmarkImagesToGetBsplineDisplacementFieldParamsDictTagged = typing.TypedDict('AntsuseLandmarkImagesToGetBsplineDisplacementFieldParamsDictTagged', {
    "@type": typing.Literal["ants/ANTSUseLandmarkImagesToGetBSplineDisplacementField"],
    "fixed_image_with_labeled_landmarks": InputPathType,
    "moving_image_with_labeled_landmarks": InputPathType,
    "output_displacement_field": str,
    "mesh_size": str,
    "number_of_levels": int,
    "order": typing.NotRequired[int | None],
    "enforce_stationary_boundaries": typing.NotRequired[int | None],
    "landmark_weights": typing.NotRequired[InputPathType | None],
})
AntsuseLandmarkImagesToGetBsplineDisplacementFieldParamsDict = _AntsuseLandmarkImagesToGetBsplineDisplacementFieldParamsDictNoTag | AntsuseLandmarkImagesToGetBsplineDisplacementFieldParamsDictTagged


class AntsuseLandmarkImagesToGetBsplineDisplacementFieldOutputs(typing.NamedTuple):
    """
    Output object returned when calling `AntsuseLandmarkImagesToGetBsplineDisplacementFieldParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    displacement_field: OutputPathType
    """The resultant displacement field for the registration process."""


def antsuse_landmark_images_to_get_bspline_displacement_field_params(
    fixed_image_with_labeled_landmarks: InputPathType,
    moving_image_with_labeled_landmarks: InputPathType,
    output_displacement_field: str,
    mesh_size: str,
    number_of_levels: int,
    order: int | None = None,
    enforce_stationary_boundaries: int | None = None,
    landmark_weights: InputPathType | None = None,
) -> AntsuseLandmarkImagesToGetBsplineDisplacementFieldParamsDictTagged:
    """
    Build parameters.
    
    Args:
        fixed_image_with_labeled_landmarks: The fixed image with labeled\
            landmarks.
        moving_image_with_labeled_landmarks: The moving image with labeled\
            landmarks.
        output_displacement_field: The output displacement field file.
        mesh_size: The mesh size specified as meshSize[0]xmeshSize[1]x...
        number_of_levels: The number of levels in the B-spline hierarchy.
        order: The order of the B-spline (default is 3).
        enforce_stationary_boundaries: Whether to enforce stationary boundaries\
            (default is 1).
        landmark_weights: Optional text file containing landmark weights. Each\
            row is either "label,labelWeight" or "labelWeight". If the latter\
            format is used, the weights are assumed to be arranged in ascending\
            order by label.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "ants/ANTSUseLandmarkImagesToGetBSplineDisplacementField",
        "fixed_image_with_labeled_landmarks": fixed_image_with_labeled_landmarks,
        "moving_image_with_labeled_landmarks": moving_image_with_labeled_landmarks,
        "output_displacement_field": output_displacement_field,
        "mesh_size": mesh_size,
        "number_of_levels": number_of_levels,
    }
    if order is not None:
        params["order"] = order
    if enforce_stationary_boundaries is not None:
        params["enforce_stationary_boundaries"] = enforce_stationary_boundaries
    if landmark_weights is not None:
        params["landmark_weights"] = landmark_weights
    return params


def antsuse_landmark_images_to_get_bspline_displacement_field_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `AntsuseLandmarkImagesToGetBsplineDisplacementFieldParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("fixed_image_with_labeled_landmarks", None) is None:
        raise StyxValidationError("`fixed_image_with_labeled_landmarks` must not be None")
    if not isinstance(params["fixed_image_with_labeled_landmarks"], (pathlib.Path, str)):
        raise StyxValidationError(f'`fixed_image_with_labeled_landmarks` has the wrong type: Received `{type(params.get("fixed_image_with_labeled_landmarks", None))}` expected `InputPathType`')
    if params.get("moving_image_with_labeled_landmarks", None) is None:
        raise StyxValidationError("`moving_image_with_labeled_landmarks` must not be None")
    if not isinstance(params["moving_image_with_labeled_landmarks"], (pathlib.Path, str)):
        raise StyxValidationError(f'`moving_image_with_labeled_landmarks` has the wrong type: Received `{type(params.get("moving_image_with_labeled_landmarks", None))}` expected `InputPathType`')
    if params.get("output_displacement_field", None) is None:
        raise StyxValidationError("`output_displacement_field` must not be None")
    if not isinstance(params["output_displacement_field"], str):
        raise StyxValidationError(f'`output_displacement_field` has the wrong type: Received `{type(params.get("output_displacement_field", None))}` expected `str`')
    if params.get("mesh_size", None) is None:
        raise StyxValidationError("`mesh_size` must not be None")
    if not isinstance(params["mesh_size"], str):
        raise StyxValidationError(f'`mesh_size` has the wrong type: Received `{type(params.get("mesh_size", None))}` expected `str`')
    if params.get("number_of_levels", None) is None:
        raise StyxValidationError("`number_of_levels` must not be None")
    if not isinstance(params["number_of_levels"], int):
        raise StyxValidationError(f'`number_of_levels` has the wrong type: Received `{type(params.get("number_of_levels", None))}` expected `int`')
    if params.get("order", None) is not None:
        if not isinstance(params["order"], int):
            raise StyxValidationError(f'`order` has the wrong type: Received `{type(params.get("order", None))}` expected `int | None`')
    if params.get("enforce_stationary_boundaries", None) is not None:
        if not isinstance(params["enforce_stationary_boundaries"], int):
            raise StyxValidationError(f'`enforce_stationary_boundaries` has the wrong type: Received `{type(params.get("enforce_stationary_boundaries", None))}` expected `int | None`')
    if params.get("landmark_weights", None) is not None:
        if not isinstance(params["landmark_weights"], (pathlib.Path, str)):
            raise StyxValidationError(f'`landmark_weights` has the wrong type: Received `{type(params.get("landmark_weights", None))}` expected `InputPathType | None`')


def antsuse_landmark_images_to_get_bspline_displacement_field_cargs(
    params: AntsuseLandmarkImagesToGetBsplineDisplacementFieldParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("ANTSUseLandmarkImagesToGetBSplineDisplacementField")
    cargs.append(execution.input_file(params.get("fixed_image_with_labeled_landmarks", None)))
    cargs.append(execution.input_file(params.get("moving_image_with_labeled_landmarks", None)))
    cargs.append(params.get("output_displacement_field", None))
    cargs.append(params.get("mesh_size", None))
    cargs.append(str(params.get("number_of_levels", None)))
    if params.get("order", None) is not None:
        cargs.append(str(params.get("order", None)))
    if params.get("enforce_stationary_boundaries", None) is not None:
        cargs.append(str(params.get("enforce_stationary_boundaries", None)))
    if params.get("landmark_weights", None) is not None:
        cargs.append(execution.input_file(params.get("landmark_weights", None)))
    return cargs


def antsuse_landmark_images_to_get_bspline_displacement_field_outputs(
    params: AntsuseLandmarkImagesToGetBsplineDisplacementFieldParamsDict,
    execution: Execution,
) -> AntsuseLandmarkImagesToGetBsplineDisplacementFieldOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = AntsuseLandmarkImagesToGetBsplineDisplacementFieldOutputs(
        root=execution.output_file("."),
        displacement_field=execution.output_file(params.get("output_displacement_field", None)),
    )
    return ret


def antsuse_landmark_images_to_get_bspline_displacement_field_execute(
    params: AntsuseLandmarkImagesToGetBsplineDisplacementFieldParamsDict,
    runner: Runner | None = None,
) -> AntsuseLandmarkImagesToGetBsplineDisplacementFieldOutputs:
    """
    ANTSUseLandmarkImagesToGetBSplineDisplacementField
    
    We expect the input images to be (1) N-ary (2) in the same physical space as
    the images you want to register and (3) to have the same landmark points
    defined within them. Landmarks will be defined from the center of mass of
    the labels in the input images. You can use ITK-snap to generate the label
    images. The optional landmarks weights are read from a text file where each
    row is either: "label,labelWeight" or "labelWeight". If the latter format is
    used, the label weights are assumed to be arranged in ascending order by
    label.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `AntsuseLandmarkImagesToGetBsplineDisplacementFieldOutputs`).
    """
    antsuse_landmark_images_to_get_bspline_displacement_field_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(ANTSUSE_LANDMARK_IMAGES_TO_GET_BSPLINE_DISPLACEMENT_FIELD_METADATA)
    params = execution.params(params)
    cargs = antsuse_landmark_images_to_get_bspline_displacement_field_cargs(params, execution)
    ret = antsuse_landmark_images_to_get_bspline_displacement_field_outputs(params, execution)
    execution.run(cargs)
    return ret


def antsuse_landmark_images_to_get_bspline_displacement_field(
    fixed_image_with_labeled_landmarks: InputPathType,
    moving_image_with_labeled_landmarks: InputPathType,
    output_displacement_field: str,
    mesh_size: str,
    number_of_levels: int,
    order: int | None = None,
    enforce_stationary_boundaries: int | None = None,
    landmark_weights: InputPathType | None = None,
    runner: Runner | None = None,
) -> AntsuseLandmarkImagesToGetBsplineDisplacementFieldOutputs:
    """
    ANTSUseLandmarkImagesToGetBSplineDisplacementField
    
    We expect the input images to be (1) N-ary (2) in the same physical space as
    the images you want to register and (3) to have the same landmark points
    defined within them. Landmarks will be defined from the center of mass of
    the labels in the input images. You can use ITK-snap to generate the label
    images. The optional landmarks weights are read from a text file where each
    row is either: "label,labelWeight" or "labelWeight". If the latter format is
    used, the label weights are assumed to be arranged in ascending order by
    label.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        fixed_image_with_labeled_landmarks: The fixed image with labeled\
            landmarks.
        moving_image_with_labeled_landmarks: The moving image with labeled\
            landmarks.
        output_displacement_field: The output displacement field file.
        mesh_size: The mesh size specified as meshSize[0]xmeshSize[1]x...
        number_of_levels: The number of levels in the B-spline hierarchy.
        order: The order of the B-spline (default is 3).
        enforce_stationary_boundaries: Whether to enforce stationary boundaries\
            (default is 1).
        landmark_weights: Optional text file containing landmark weights. Each\
            row is either "label,labelWeight" or "labelWeight". If the latter\
            format is used, the weights are assumed to be arranged in ascending\
            order by label.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `AntsuseLandmarkImagesToGetBsplineDisplacementFieldOutputs`).
    """
    params = antsuse_landmark_images_to_get_bspline_displacement_field_params(
        fixed_image_with_labeled_landmarks=fixed_image_with_labeled_landmarks,
        moving_image_with_labeled_landmarks=moving_image_with_labeled_landmarks,
        output_displacement_field=output_displacement_field,
        mesh_size=mesh_size,
        number_of_levels=number_of_levels,
        order=order,
        enforce_stationary_boundaries=enforce_stationary_boundaries,
        landmark_weights=landmark_weights,
    )
    return antsuse_landmark_images_to_get_bspline_displacement_field_execute(params, runner)


__all__ = [
    "ANTSUSE_LANDMARK_IMAGES_TO_GET_BSPLINE_DISPLACEMENT_FIELD_METADATA",
    "AntsuseLandmarkImagesToGetBsplineDisplacementFieldOutputs",
    "AntsuseLandmarkImagesToGetBsplineDisplacementFieldParamsDict",
    "AntsuseLandmarkImagesToGetBsplineDisplacementFieldParamsDictTagged",
    "antsuse_landmark_images_to_get_bspline_displacement_field",
    "antsuse_landmark_images_to_get_bspline_displacement_field_execute",
    "antsuse_landmark_images_to_get_bspline_displacement_field_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

ANTS_MULTIVARIATE_TEMPLATE_CONSTRUCTION2_SH_METADATA = Metadata(
    id="dcd5ca5151722ef03dfc3e47fb8b190e2c1c0932.boutiques",
    name="antsMultivariateTemplateConstruction2.sh",
    package="ants",
    container_image_tag="antsx/ants:v2.5.3",
)


_AntsMultivariateTemplateConstruction2ShParamsDictNoTag = typing.TypedDict('_AntsMultivariateTemplateConstruction2ShParamsDictNoTag', {
    "dimension": typing.Literal[2, 3, 4],
    "output_prefix": typing.NotRequired[str | None],
    "image_statistic": typing.NotRequired[typing.Literal[0, 1, 2] | None],
    "sharpening": typing.NotRequired[typing.Literal[0, 1, 2] | None],
    "backup_images": typing.NotRequired[bool | None],
    "parallel_control": typing.NotRequired[typing.Literal[0, 1, 2, 3, 4, 5] | None],
    "single_precision": typing.NotRequired[bool | None],
    "gradient_step": typing.NotRequired[float | None],
    "iterations": typing.NotRequired[int | None],
    "cpu_cores": typing.NotRequired[int | None],
    "num_modalities": typing.NotRequired[int | None],
    "modality_weights": typing.NotRequired[str | None],
    "max_iterations": typing.NotRequired[str | None],
    "shrink_factors": typing.NotRequired[str | None],
    "smoothing_kernels": typing.NotRequired[str | None],
    "n4_bias_correction": typing.NotRequired[bool | None],
    "prepend_commands": typing.NotRequired[str | None],
    "rigid_registration": typing.NotRequired[bool | None],
    "linear_registration": typing.NotRequired[bool | None],
    "similarity_metric": typing.NotRequired[str | None],
    "transformation_type": typing.NotRequired[str | None],
    "walltime": typing.NotRequired[str | None],
    "memory_limit": typing.NotRequired[str | None],
    "xgrid_args": typing.NotRequired[str | None],
    "update_template": typing.NotRequired[bool | None],
    "target_volume": typing.NotRequired[list[InputPathType] | None],
    "input_images": list[InputPathType],
})
AntsMultivariateTemplateConstruction2ShParamsDictTagged = typing.TypedDict('AntsMultivariateTemplateConstruction2ShParamsDictTagged', {
    "@type": typing.Literal["ants/antsMultivariateTemplateConstruction2.sh"],
    "dimension": typing.Literal[2, 3, 4],
    "output_prefix": typing.NotRequired[str | None],
    "image_statistic": typing.NotRequired[typing.Literal[0, 1, 2] | None],
    "sharpening": typing.NotRequired[typing.Literal[0, 1, 2] | None],
    "backup_images": typing.NotRequired[bool | None],
    "parallel_control": typing.NotRequired[typing.Literal[0, 1, 2, 3, 4, 5] | None],
    "single_precision": typing.NotRequired[bool | None],
    "gradient_step": typing.NotRequired[float | None],
    "iterations": typing.NotRequired[int | None],
    "cpu_cores": typing.NotRequired[int | None],
    "num_modalities": typing.NotRequired[int | None],
    "modality_weights": typing.NotRequired[str | None],
    "max_iterations": typing.NotRequired[str | None],
    "shrink_factors": typing.NotRequired[str | None],
    "smoothing_kernels": typing.NotRequired[str | None],
    "n4_bias_correction": typing.NotRequired[bool | None],
    "prepend_commands": typing.NotRequired[str | None],
    "rigid_registration": typing.NotRequired[bool | None],
    "linear_registration": typing.NotRequired[bool | None],
    "similarity_metric": typing.NotRequired[str | None],
    "transformation_type": typing.NotRequired[str | None],
    "walltime": typing.NotRequired[str | None],
    "memory_limit": typing.NotRequired[str | None],
    "xgrid_args": typing.NotRequired[str | None],
    "update_template": typing.NotRequired[bool | None],
    "target_volume": typing.NotRequired[list[InputPathType] | None],
    "input_images": list[InputPathType],
})
AntsMultivariateTemplateConstruction2ShParamsDict = _AntsMultivariateTemplateConstruction2ShParamsDictNoTag | AntsMultivariateTemplateConstruction2ShParamsDictTagged


class AntsMultivariateTemplateConstruction2ShOutputs(typing.NamedTuple):
    """
    Output object returned when calling `AntsMultivariateTemplateConstruction2ShParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    intermediate_templates_dir: OutputPathType
    """Directory containing intermediate templates and shape update warps from
    each iteration"""


def ants_multivariate_template_construction2_sh_params(
    dimension: typing.Literal[2, 3, 4],
    input_images: list[InputPathType],
    output_prefix: str | None = None,
    image_statistic: typing.Literal[0, 1, 2] | None = None,
    sharpening: typing.Literal[0, 1, 2] | None = None,
    backup_images: bool | None = None,
    parallel_control: typing.Literal[0, 1, 2, 3, 4, 5] | None = None,
    single_precision: bool | None = None,
    gradient_step: float | None = None,
    iterations: int | None = None,
    cpu_cores: int | None = None,
    num_modalities: int | None = None,
    modality_weights: str | None = None,
    max_iterations: str | None = None,
    shrink_factors: str | None = None,
    smoothing_kernels: str | None = None,
    n4_bias_correction: bool | None = None,
    prepend_commands: str | None = None,
    rigid_registration: bool | None = None,
    linear_registration: bool | None = None,
    similarity_metric: str | None = None,
    transformation_type: str | None = None,
    walltime: str | None = None,
    memory_limit: str | None = None,
    xgrid_args: str | None = None,
    update_template: bool | None = None,
    target_volume: list[InputPathType] | None = None,
) -> AntsMultivariateTemplateConstruction2ShParamsDictTagged:
    """
    Build parameters.
    
    Args:
        dimension: ImageDimension: 2 or 3 (for 2 or 3 dimensional registration\
            of single volume), 4 (for template generation of time-series data).
        input_images: List of images in the current directory, eg *_t1.nii.gz.\
            Should be at the end of the command. Optionally, one can specify a .csv\
            or .txt file where each line is the location of the input image. One\
            can also specify more than one file for each image for multi-modal\
            template construction (e.g. t1 and t2). For the multi-modal case, the\
            templates will be consecutively numbered.
        output_prefix: A prefix that is prepended to all output files (default\
            = "antsBTP").
        image_statistic: Image statistic used to summarize images (default 1):\
            0 = mean, 1 = mean of normalized intensities, 2 = median. Normalization\
            here means dividing each image by its mean intensity.
        sharpening: Sharpening applied to template at each iteration (default\
            1): 0 = none, 1 = Laplacian, 2 = Unsharp mask.
        backup_images: Backup images and results from all iterations (default =\
            0): Boolean to save the transform files, bias corrected inputs,\
            templates, transforms, and warped images for each iteration. By\
            default, only the templates and the shape update warp field are saved.
        parallel_control: Control for parallel computation (default 0): 0 = run\
            serially, 1 = SGE qsub, 2 = use PEXEC (localhost), 3 = Apple XGrid, 4 =\
            PBS qsub, 5 = SLURM.
        single_precision: Use single precision (default 1).
        gradient_step: Gradient step size (default 0.25): smaller in magnitude\
            results in more cautious steps. Use smaller steps to refine template\
            details. 0.25 is an upper (aggressive) limit for this parameter.
        iterations: Iteration limit (default 4): iterations of the template\
            construction (Iteration limit)*NumImages registrations.
        cpu_cores: Number of cpu cores to use locally for pexec option (default\
            2; requires "-c 2").
        num_modalities: Number of modalities used to construct the template\
            (default 1): For example, if one wanted to create a multimodal template\
            consisting of T1,T2,and FA components ("-k 3").
        modality_weights: Modality weights used in the similarity metric\
            (default = 1): specified as e.g. 1x0.5x0.75.
        max_iterations: Max iterations for each pairwise registration (default\
            = 100x100x70x20): specified in the form JxK...xF where J = max\
            iterations at first (coarsest) resolution, K = max iterations at next\
            resolution, F = max iterations at the final resolution.
        shrink_factors: Shrink factors in pairwise registration (default =\
            6x4x2x1): in the same form as max iterations. Must have the same number\
            of components as the iterations and smoothing. The shrink factors are\
            integer factors for downsampling the virtual space (usually the\
            template image) during registration.
        smoothing_kernels: Smoothing kernels in pairwise registration (default\
            = 3x2x1x0): also in the same form as max iterations and shrink factors,\
            with the same number of components. Standard deviation of a Gaussian\
            smoothing kernel applied to the images before downsampling at each\
            level. The kernel may be specified in mm units or voxels with "AxBxCmm"\
            or "AxBxCvox". Missing units implies vox.
        n4_bias_correction: N4BiasFieldCorrection of moving image: 0 off, 1 on\
            (default 1).
        prepend_commands: Commands to prepend to job scripts (e.g., change into\
            appropriate directory, set paths, etc).
        rigid_registration: Do rigid-body registration of inputs to the initial\
            template, before doing the main pairwise registration. 0 off 1 on\
            (default 0). If you are trying to refine or update an existing\
            template, you would use '-r 0'. Rigid initialization is useful when you\
            do not have an initial template, or you want to use a single image as a\
            reference for rigid alignment only.
        linear_registration: Use linear image registration stages during the\
            pairwise (template/subject) deformable registration. Otherwise,\
            registration is limited to SyN or B-spline SyN (see '-t' option). This\
            is '1' by default.
        similarity_metric: Type of similarity metric used for pairwise\
            registration (default = CC). Options are case sensitive. CC =\
            cross-correlation, MI = mutual information, MSQ = mean square\
            difference, DEMONS = demon's metric. A similarity metric per modality\
            can be specified. If the CC metric is chosen, one can also specify the\
            radius in brackets, e.g. '-m "CC[4]"'.
        transformation_type: Type of transformation model used for registration\
            (default = SyN): Options are case sensitive. SyN = Greedy SyN,\
            BSplineSyN = Greedy B-spline SyN, TimeVaryingVelocityField =\
            Time-varying velocity field, TimeVaryingBSplineVelocityField =\
            Time-varying B-spline velocity field. Transformation parameters may be\
            specified with brackets, eg '-t "SyN[0.1,3,0]"'. To use linear\
            registration only, options are: Affine = Affine (runs Rigid + Affine),\
            Rigid = Rigid (runs Rigid only).
        walltime: Walltime (default = 20:00:00): Option for PBS/SLURM qsub\
            specifying requested time per pairwise registration.
        memory_limit: Memory limit (default = 8gb): Option for PBS/SLURM qsub\
            specifying requested memory per pairwise registration.
        xgrid_args: XGrid arguments (e.g., -x "-p password -h controlhost").
        update_template: Update the template with the full affine transform\
            (default 1). If 0, the rigid component of the affine transform will not\
            be used to update the template. If your template drifts in translation\
            or orientation try "-y 0".
        target_volume: Use this this volume as the target of all inputs. When\
            not used, the script will create an unbiased starting point by\
            averaging all inputs, then aligning the center of mass of all inputs to\
            that of the initial average. If you do not use -z, it is recommended to\
            use "-r 1". Use the full path. For multiple modalities, specify -z\
            modality1.nii.gz -z modality2.nii.gz ... in the same modality order as\
            the input images.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "ants/antsMultivariateTemplateConstruction2.sh",
        "dimension": dimension,
        "input_images": input_images,
    }
    if output_prefix is not None:
        params["output_prefix"] = output_prefix
    if image_statistic is not None:
        params["image_statistic"] = image_statistic
    if sharpening is not None:
        params["sharpening"] = sharpening
    if backup_images is not None:
        params["backup_images"] = backup_images
    if parallel_control is not None:
        params["parallel_control"] = parallel_control
    if single_precision is not None:
        params["single_precision"] = single_precision
    if gradient_step is not None:
        params["gradient_step"] = gradient_step
    if iterations is not None:
        params["iterations"] = iterations
    if cpu_cores is not None:
        params["cpu_cores"] = cpu_cores
    if num_modalities is not None:
        params["num_modalities"] = num_modalities
    if modality_weights is not None:
        params["modality_weights"] = modality_weights
    if max_iterations is not None:
        params["max_iterations"] = max_iterations
    if shrink_factors is not None:
        params["shrink_factors"] = shrink_factors
    if smoothing_kernels is not None:
        params["smoothing_kernels"] = smoothing_kernels
    if n4_bias_correction is not None:
        params["n4_bias_correction"] = n4_bias_correction
    if prepend_commands is not None:
        params["prepend_commands"] = prepend_commands
    if rigid_registration is not None:
        params["rigid_registration"] = rigid_registration
    if linear_registration is not None:
        params["linear_registration"] = linear_registration
    if similarity_metric is not None:
        params["similarity_metric"] = similarity_metric
    if transformation_type is not None:
        params["transformation_type"] = transformation_type
    if walltime is not None:
        params["walltime"] = walltime
    if memory_limit is not None:
        params["memory_limit"] = memory_limit
    if xgrid_args is not None:
        params["xgrid_args"] = xgrid_args
    if update_template is not None:
        params["update_template"] = update_template
    if target_volume is not None:
        params["target_volume"] = target_volume
    return params


def ants_multivariate_template_construction2_sh_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `AntsMultivariateTemplateConstruction2ShParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("dimension", None) is None:
        raise StyxValidationError("`dimension` must not be None")
    if not isinstance(params["dimension"], int):
        raise StyxValidationError(f'`dimension` has the wrong type: Received `{type(params.get("dimension", None))}` expected `typing.Literal[2, 3, 4]`')
    if params["dimension"] not in [2, 3, 4]:
        raise StyxValidationError("Parameter `dimension` must be one of [2, 3, 4]")
    if params.get("output_prefix", None) is not None:
        if not isinstance(params["output_prefix"], str):
            raise StyxValidationError(f'`output_prefix` has the wrong type: Received `{type(params.get("output_prefix", None))}` expected `str | None`')
    if params.get("image_statistic", None) is not None:
        if not isinstance(params["image_statistic"], int):
            raise StyxValidationError(f'`image_statistic` has the wrong type: Received `{type(params.get("image_statistic", None))}` expected `typing.Literal[0, 1, 2] | None`')
        if params["image_statistic"] not in [0, 1, 2]:
            raise StyxValidationError("Parameter `image_statistic` must be one of [0, 1, 2]")
    if params.get("sharpening", None) is not None:
        if not isinstance(params["sharpening"], int):
            raise StyxValidationError(f'`sharpening` has the wrong type: Received `{type(params.get("sharpening", None))}` expected `typing.Literal[0, 1, 2] | None`')
        if params["sharpening"] not in [0, 1, 2]:
            raise StyxValidationError("Parameter `sharpening` must be one of [0, 1, 2]")
    if params.get("backup_images", None) is not None:
        if not isinstance(params["backup_images"], bool):
            raise StyxValidationError(f'`backup_images` has the wrong type: Received `{type(params.get("backup_images", None))}` expected `bool | None`')
    if params.get("parallel_control", None) is not None:
        if not isinstance(params["parallel_control"], int):
            raise StyxValidationError(f'`parallel_control` has the wrong type: Received `{type(params.get("parallel_control", None))}` expected `typing.Literal[0, 1, 2, 3, 4, 5] | None`')
        if params["parallel_control"] not in [0, 1, 2, 3, 4, 5]:
            raise StyxValidationError("Parameter `parallel_control` must be one of [0, 1, 2, 3, 4, 5]")
    if params.get("single_precision", None) is not None:
        if not isinstance(params["single_precision"], bool):
            raise StyxValidationError(f'`single_precision` has the wrong type: Received `{type(params.get("single_precision", None))}` expected `bool | None`')
    if params.get("gradient_step", None) is not None:
        if not isinstance(params["gradient_step"], (float, int)):
            raise StyxValidationError(f'`gradient_step` has the wrong type: Received `{type(params.get("gradient_step", None))}` expected `float | None`')
        if params["gradient_step"] > 0.25:
            raise StyxValidationError("Parameter `gradient_step` must be at most 0.25")
    if params.get("iterations", None) is not None:
        if not isinstance(params["iterations"], int):
            raise StyxValidationError(f'`iterations` has the wrong type: Received `{type(params.get("iterations", None))}` expected `int | None`')
    if params.get("cpu_cores", None) is not None:
        if not isinstance(params["cpu_cores"], int):
            raise StyxValidationError(f'`cpu_cores` has the wrong type: Received `{type(params.get("cpu_cores", None))}` expected `int | None`')
    if params.get("num_modalities", None) is not None:
        if not isinstance(params["num_modalities"], int):
            raise StyxValidationError(f'`num_modalities` has the wrong type: Received `{type(params.get("num_modalities", None))}` expected `int | None`')
    if params.get("modality_weights", None) is not None:
        if not isinstance(params["modality_weights"], str):
            raise StyxValidationError(f'`modality_weights` has the wrong type: Received `{type(params.get("modality_weights", None))}` expected `str | None`')
    if params.get("max_iterations", None) is not None:
        if not isinstance(params["max_iterations"], str):
            raise StyxValidationError(f'`max_iterations` has the wrong type: Received `{type(params.get("max_iterations", None))}` expected `str | None`')
    if params.get("shrink_factors", None) is not None:
        if not isinstance(params["shrink_factors"], str):
            raise StyxValidationError(f'`shrink_factors` has the wrong type: Received `{type(params.get("shrink_factors", None))}` expected `str | None`')
    if params.get("smoothing_kernels", None) is not None:
        if not isinstance(params["smoothing_kernels"], str):
            raise StyxValidationError(f'`smoothing_kernels` has the wrong type: Received `{type(params.get("smoothing_kernels", None))}` expected `str | None`')
    if params.get("n4_bias_correction", None) is not None:
        if not isinstance(params["n4_bias_correction"], bool):
            raise StyxValidationError(f'`n4_bias_correction` has the wrong type: Received `{type(params.get("n4_bias_correction", None))}` expected `bool | None`')
    if params.get("prepend_commands", None) is not None:
        if not isinstance(params["prepend_commands"], str):
            raise StyxValidationError(f'`prepend_commands` has the wrong type: Received `{type(params.get("prepend_commands", None))}` expected `str | None`')
    if params.get("rigid_registration", None) is not None:
        if not isinstance(params["rigid_registration"], bool):
            raise StyxValidationError(f'`rigid_registration` has the wrong type: Received `{type(params.get("rigid_registration", None))}` expected `bool | None`')
    if params.get("linear_registration", None) is not None:
        if not isinstance(params["linear_registration"], bool):
            raise StyxValidationError(f'`linear_registration` has the wrong type: Received `{type(params.get("linear_registration", None))}` expected `bool | None`')
    if params.get("similarity_metric", None) is not None:
        if not isinstance(params["similarity_metric"], str):
            raise StyxValidationError(f'`similarity_metric` has the wrong type: Received `{type(params.get("similarity_metric", None))}` expected `str | None`')
    if params.get("transformation_type", None) is not None:
        if not isinstance(params["transformation_type"], str):
            raise StyxValidationError(f'`transformation_type` has the wrong type: Received `{type(params.get("transformation_type", None))}` expected `str | None`')
    if params.get("walltime", None) is not None:
        if not isinstance(params["walltime"], str):
            raise StyxValidationError(f'`walltime` has the wrong type: Received `{type(params.get("walltime", None))}` expected `str | None`')
    if params.get("memory_limit", None) is not None:
        if not isinstance(params["memory_limit"], str):
            raise StyxValidationError(f'`memory_limit` has the wrong type: Received `{type(params.get("memory_limit", None))}` expected `str | None`')
    if params.get("xgrid_args", None) is not None:
        if not isinstance(params["xgrid_args"], str):
            raise StyxValidationError(f'`xgrid_args` has the wrong type: Received `{type(params.get("xgrid_args", None))}` expected `str | None`')
    if params.get("update_template", None) is not None:
        if not isinstance(params["update_template"], bool):
            raise StyxValidationError(f'`update_template` has the wrong type: Received `{type(params.get("update_template", None))}` expected `bool | None`')
    if params.get("target_volume", None) is not None:
        if not isinstance(params["target_volume"], list):
            raise StyxValidationError(f'`target_volume` has the wrong type: Received `{type(params.get("target_volume", None))}` expected `list[InputPathType] | None`')
        for e in params["target_volume"]:
            if not isinstance(e, (pathlib.Path, str)):
                raise StyxValidationError(f'`target_volume` has the wrong type: Received `{type(params.get("target_volume", None))}` expected `list[InputPathType] | None`')
    if params.get("input_images", None) is None:
        raise StyxValidationError("`input_images` must not be None")
    if not isinstance(params["input_images"], list):
        raise StyxValidationError(f'`input_images` has the wrong type: Received `{type(params.get("input_images", None))}` expected `list[InputPathType]`')
    for e in params["input_images"]:
        if not isinstance(e, (pathlib.Path, str)):
            raise StyxValidationError(f'`input_images` has the wrong type: Received `{type(params.get("input_images", None))}` expected `list[InputPathType]`')


def ants_multivariate_template_construction2_sh_cargs(
    params: AntsMultivariateTemplateConstruction2ShParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("antsMultivariateTemplateConstruction2.sh")
    cargs.extend([
        "-d",
        str(params.get("dimension", None))
    ])
    if params.get("output_prefix", None) is not None:
        cargs.extend([
            "-o",
            params.get("output_prefix", None)
        ])
    if params.get("image_statistic", None) is not None:
        cargs.extend([
            "-a",
            str(params.get("image_statistic", None))
        ])
    if params.get("sharpening", None) is not None:
        cargs.extend([
            "-A",
            str(params.get("sharpening", None))
        ])
    if params.get("backup_images", None) is not None:
        cargs.extend([
            "-b",
            ("1" if params.get("backup_images", None) else "0")
        ])
    if params.get("parallel_control", None) is not None:
        cargs.extend([
            "-c",
            str(params.get("parallel_control", None))
        ])
    if params.get("single_precision", None) is not None:
        cargs.extend([
            "-e",
            ("1" if params.get("single_precision", None) else "0")
        ])
    if params.get("gradient_step", None) is not None:
        cargs.extend([
            "-g",
            str(params.get("gradient_step", None))
        ])
    if params.get("iterations", None) is not None:
        cargs.extend([
            "-i",
            str(params.get("iterations", None))
        ])
    if params.get("cpu_cores", None) is not None:
        cargs.extend([
            "-j",
            str(params.get("cpu_cores", None))
        ])
    if params.get("num_modalities", None) is not None:
        cargs.extend([
            "-k",
            str(params.get("num_modalities", None))
        ])
    if params.get("modality_weights", None) is not None:
        cargs.extend([
            "-w",
            params.get("modality_weights", None)
        ])
    if params.get("max_iterations", None) is not None:
        cargs.extend([
            "-q",
            params.get("max_iterations", None)
        ])
    if params.get("shrink_factors", None) is not None:
        cargs.extend([
            "-f",
            params.get("shrink_factors", None)
        ])
    if params.get("smoothing_kernels", None) is not None:
        cargs.extend([
            "-s",
            params.get("smoothing_kernels", None)
        ])
    if params.get("n4_bias_correction", None) is not None:
        cargs.extend([
            "-n",
            ("1" if params.get("n4_bias_correction", None) else "0")
        ])
    if params.get("prepend_commands", None) is not None:
        cargs.extend([
            "-p",
            params.get("prepend_commands", None)
        ])
    if params.get("rigid_registration", None) is not None:
        cargs.extend([
            "-r",
            ("1" if params.get("rigid_registration", None) else "0")
        ])
    if params.get("linear_registration", None) is not None:
        cargs.extend([
            "-l",
            ("1" if params.get("linear_registration", None) else "0")
        ])
    if params.get("similarity_metric", None) is not None:
        cargs.extend([
            "-m",
            params.get("similarity_metric", None)
        ])
    if params.get("transformation_type", None) is not None:
        cargs.extend([
            "-t",
            params.get("transformation_type", None)
        ])
    if params.get("walltime", None) is not None:
        cargs.extend([
            "-u",
            params.get("walltime", None)
        ])
    if params.get("memory_limit", None) is not None:
        cargs.extend([
            "-v",
            params.get("memory_limit", None)
        ])
    if params.get("xgrid_args", None) is not None:
        cargs.extend([
            "-x",
            params.get("xgrid_args", None)
        ])
    if params.get("update_template", None) is not None:
        cargs.extend([
            "-y",
            ("1" if params.get("update_template", None) else "0")
        ])
    if params.get("target_volume", None) is not None:
        cargs.extend([
            "-z",
            *[execution.input_file(f) for f in params.get("target_volume", None)]
        ])
    cargs.extend([execution.input_file(f) for f in params.get("input_images", None)])
    return cargs


def ants_multivariate_template_construction2_sh_outputs(
    params: AntsMultivariateTemplateConstruction2ShParamsDict,
    execution: Execution,
) -> AntsMultivariateTemplateConstruction2ShOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = AntsMultivariateTemplateConstruction2ShOutputs(
        root=execution.output_file("."),
        intermediate_templates_dir=execution.output_file("intermediateTemplates"),
    )
    return ret


def ants_multivariate_template_construction2_sh_execute(
    params: AntsMultivariateTemplateConstruction2ShParamsDict,
    runner: Runner | None = None,
) -> AntsMultivariateTemplateConstruction2ShOutputs:
    """
    antsMultivariateTemplateConstruction2.sh
    
    Multivariate template construction using ANTs registration tools (version
    2). All images to be added to the template should be in the same directory,
    and this script should be invoked from that directory.
    
    Author: Brian B. Avants, Nick Tustison and Gang Song
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `AntsMultivariateTemplateConstruction2ShOutputs`).
    """
    ants_multivariate_template_construction2_sh_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(ANTS_MULTIVARIATE_TEMPLATE_CONSTRUCTION2_SH_METADATA)
    params = execution.params(params)
    cargs = ants_multivariate_template_construction2_sh_cargs(params, execution)
    ret = ants_multivariate_template_construction2_sh_outputs(params, execution)
    execution.run(cargs)
    return ret


def ants_multivariate_template_construction2_sh(
    dimension: typing.Literal[2, 3, 4],
    input_images: list[InputPathType],
    output_prefix: str | None = None,
    image_statistic: typing.Literal[0, 1, 2] | None = None,
    sharpening: typing.Literal[0, 1, 2] | None = None,
    backup_images: bool | None = None,
    parallel_control: typing.Literal[0, 1, 2, 3, 4, 5] | None = None,
    single_precision: bool | None = None,
    gradient_step: float | None = None,
    iterations: int | None = None,
    cpu_cores: int | None = None,
    num_modalities: int | None = None,
    modality_weights: str | None = None,
    max_iterations: str | None = None,
    shrink_factors: str | None = None,
    smoothing_kernels: str | None = None,
    n4_bias_correction: bool | None = None,
    prepend_commands: str | None = None,
    rigid_registration: bool | None = None,
    linear_registration: bool | None = None,
    similarity_metric: str | None = None,
    transformation_type: str | None = None,
    walltime: str | None = None,
    memory_limit: str | None = None,
    xgrid_args: str | None = None,
    update_template: bool | None = None,
    target_volume: list[InputPathType] | None = None,
    runner: Runner | None = None,
) -> AntsMultivariateTemplateConstruction2ShOutputs:
    """
    antsMultivariateTemplateConstruction2.sh
    
    Multivariate template construction using ANTs registration tools (version
    2). All images to be added to the template should be in the same directory,
    and this script should be invoked from that directory.
    
    Author: Brian B. Avants, Nick Tustison and Gang Song
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        dimension: ImageDimension: 2 or 3 (for 2 or 3 dimensional registration\
            of single volume), 4 (for template generation of time-series data).
        input_images: List of images in the current directory, eg *_t1.nii.gz.\
            Should be at the end of the command. Optionally, one can specify a .csv\
            or .txt file where each line is the location of the input image. One\
            can also specify more than one file for each image for multi-modal\
            template construction (e.g. t1 and t2). For the multi-modal case, the\
            templates will be consecutively numbered.
        output_prefix: A prefix that is prepended to all output files (default\
            = "antsBTP").
        image_statistic: Image statistic used to summarize images (default 1):\
            0 = mean, 1 = mean of normalized intensities, 2 = median. Normalization\
            here means dividing each image by its mean intensity.
        sharpening: Sharpening applied to template at each iteration (default\
            1): 0 = none, 1 = Laplacian, 2 = Unsharp mask.
        backup_images: Backup images and results from all iterations (default =\
            0): Boolean to save the transform files, bias corrected inputs,\
            templates, transforms, and warped images for each iteration. By\
            default, only the templates and the shape update warp field are saved.
        parallel_control: Control for parallel computation (default 0): 0 = run\
            serially, 1 = SGE qsub, 2 = use PEXEC (localhost), 3 = Apple XGrid, 4 =\
            PBS qsub, 5 = SLURM.
        single_precision: Use single precision (default 1).
        gradient_step: Gradient step size (default 0.25): smaller in magnitude\
            results in more cautious steps. Use smaller steps to refine template\
            details. 0.25 is an upper (aggressive) limit for this parameter.
        iterations: Iteration limit (default 4): iterations of the template\
            construction (Iteration limit)*NumImages registrations.
        cpu_cores: Number of cpu cores to use locally for pexec option (default\
            2; requires "-c 2").
        num_modalities: Number of modalities used to construct the template\
            (default 1): For example, if one wanted to create a multimodal template\
            consisting of T1,T2,and FA components ("-k 3").
        modality_weights: Modality weights used in the similarity metric\
            (default = 1): specified as e.g. 1x0.5x0.75.
        max_iterations: Max iterations for each pairwise registration (default\
            = 100x100x70x20): specified in the form JxK...xF where J = max\
            iterations at first (coarsest) resolution, K = max iterations at next\
            resolution, F = max iterations at the final resolution.
        shrink_factors: Shrink factors in pairwise registration (default =\
            6x4x2x1): in the same form as max iterations. Must have the same number\
            of components as the iterations and smoothing. The shrink factors are\
            integer factors for downsampling the virtual space (usually the\
            template image) during registration.
        smoothing_kernels: Smoothing kernels in pairwise registration (default\
            = 3x2x1x0): also in the same form as max iterations and shrink factors,\
            with the same number of components. Standard deviation of a Gaussian\
            smoothing kernel applied to the images before downsampling at each\
            level. The kernel may be specified in mm units or voxels with "AxBxCmm"\
            or "AxBxCvox". Missing units implies vox.
        n4_bias_correction: N4BiasFieldCorrection of moving image: 0 off, 1 on\
            (default 1).
        prepend_commands: Commands to prepend to job scripts (e.g., change into\
            appropriate directory, set paths, etc).
        rigid_registration: Do rigid-body registration of inputs to the initial\
            template, before doing the main pairwise registration. 0 off 1 on\
            (default 0). If you are trying to refine or update an existing\
            template, you would use '-r 0'. Rigid initialization is useful when you\
            do not have an initial template, or you want to use a single image as a\
            reference for rigid alignment only.
        linear_registration: Use linear image registration stages during the\
            pairwise (template/subject) deformable registration. Otherwise,\
            registration is limited to SyN or B-spline SyN (see '-t' option). This\
            is '1' by default.
        similarity_metric: Type of similarity metric used for pairwise\
            registration (default = CC). Options are case sensitive. CC =\
            cross-correlation, MI = mutual information, MSQ = mean square\
            difference, DEMONS = demon's metric. A similarity metric per modality\
            can be specified. If the CC metric is chosen, one can also specify the\
            radius in brackets, e.g. '-m "CC[4]"'.
        transformation_type: Type of transformation model used for registration\
            (default = SyN): Options are case sensitive. SyN = Greedy SyN,\
            BSplineSyN = Greedy B-spline SyN, TimeVaryingVelocityField =\
            Time-varying velocity field, TimeVaryingBSplineVelocityField =\
            Time-varying B-spline velocity field. Transformation parameters may be\
            specified with brackets, eg '-t "SyN[0.1,3,0]"'. To use linear\
            registration only, options are: Affine = Affine (runs Rigid + Affine),\
            Rigid = Rigid (runs Rigid only).
        walltime: Walltime (default = 20:00:00): Option for PBS/SLURM qsub\
            specifying requested time per pairwise registration.
        memory_limit: Memory limit (default = 8gb): Option for PBS/SLURM qsub\
            specifying requested memory per pairwise registration.
        xgrid_args: XGrid arguments (e.g., -x "-p password -h controlhost").
        update_template: Update the template with the full affine transform\
            (default 1). If 0, the rigid component of the affine transform will not\
            be used to update the template. If your template drifts in translation\
            or orientation try "-y 0".
        target_volume: Use this this volume as the target of all inputs. When\
            not used, the script will create an unbiased starting point by\
            averaging all inputs, then aligning the center of mass of all inputs to\
            that of the initial average. If you do not use -z, it is recommended to\
            use "-r 1". Use the full path. For multiple modalities, specify -z\
            modality1.nii.gz -z modality2.nii.gz ... in the same modality order as\
            the input images.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `AntsMultivariateTemplateConstruction2ShOutputs`).
    """
    params = ants_multivariate_template_construction2_sh_params(
        dimension=dimension,
        output_prefix=output_prefix,
        image_statistic=image_statistic,
        sharpening=sharpening,
        backup_images=backup_images,
        parallel_control=parallel_control,
        single_precision=single_precision,
        gradient_step=gradient_step,
        iterations=iterations,
        cpu_cores=cpu_cores,
        num_modalities=num_modalities,
        modality_weights=modality_weights,
        max_iterations=max_iterations,
        shrink_factors=shrink_factors,
        smoothing_kernels=smoothing_kernels,
        n4_bias_correction=n4_bias_correction,
        prepend_commands=prepend_commands,
        rigid_registration=rigid_registration,
        linear_registration=linear_registration,
        similarity_metric=similarity_metric,
        transformation_type=transformation_type,
        walltime=walltime,
        memory_limit=memory_limit,
        xgrid_args=xgrid_args,
        update_template=update_template,
        target_volume=target_volume,
        input_images=input_images,
    )
    return ants_multivariate_template_construction2_sh_execute(params, runner)


__all__ = [
    "ANTS_MULTIVARIATE_TEMPLATE_CONSTRUCTION2_SH_METADATA",
    "AntsMultivariateTemplateConstruction2ShOutputs",
    "AntsMultivariateTemplateConstruction2ShParamsDict",
    "AntsMultivariateTemplateConstruction2ShParamsDictTagged",
    "ants_multivariate_template_construction2_sh",
    "ants_multivariate_template_construction2_sh_execute",
    "ants_multivariate_template_construction2_sh_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

CONVERT_TRANSFORM_FILE_METADATA = Metadata(
    id="09238b9afb3b35db70adc70f6459e5edc8963971.boutiques",
    name="ConvertTransformFile",
    package="ants",
    container_image_tag="antsx/ants:v2.5.3",
)


_ConvertTransformFileParamsDictNoTag = typing.TypedDict('_ConvertTransformFileParamsDictNoTag', {
    "dimensions": int,
    "input_transform_file": InputPathType,
    "output_transform_file": str,
    "matrix": bool,
    "homogeneous_matrix": bool,
    "RAS": bool,
    "convert_to_affine_type": bool,
})
ConvertTransformFileParamsDictTagged = typing.TypedDict('ConvertTransformFileParamsDictTagged', {
    "@type": typing.Literal["ants/ConvertTransformFile"],
    "dimensions": int,
    "input_transform_file": InputPathType,
    "output_transform_file": str,
    "matrix": bool,
    "homogeneous_matrix": bool,
    "RAS": bool,
    "convert_to_affine_type": bool,
})
ConvertTransformFileParamsDict = _ConvertTransformFileParamsDictNoTag | ConvertTransformFileParamsDictTagged


class ConvertTransformFileOutputs(typing.NamedTuple):
    """
    Output object returned when calling `ConvertTransformFileParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def convert_transform_file_params(
    dimensions: int,
    input_transform_file: InputPathType,
    output_transform_file: str,
    matrix: bool = False,
    homogeneous_matrix: bool = False,
    ras: bool = False,
    convert_to_affine_type: bool = False,
) -> ConvertTransformFileParamsDictTagged:
    """
    Build parameters.
    
    Args:
        dimensions: Dimensionality of the transform file.
        input_transform_file: Path to the input transform file.
        output_transform_file: Path for the output transform file.
        matrix: Output only the transform matrix to a text file, one row per\
            line with space-delimited values. Only works for specific transform\
            types.
        homogeneous_matrix: Output an N+1 square homogeneous matrix from the\
            transform matrix and offset. Only works for specific transform types.
        ras: Convert the output into the RAS coordinate system if combined with\
            'matrix' or 'homogeneousMatrix'.
        convert_to_affine_type: Convert the input transform type to\
            AffineTransform and output again as a binary transform file.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "ants/ConvertTransformFile",
        "dimensions": dimensions,
        "input_transform_file": input_transform_file,
        "output_transform_file": output_transform_file,
        "matrix": matrix,
        "homogeneous_matrix": homogeneous_matrix,
        "RAS": ras,
        "convert_to_affine_type": convert_to_affine_type,
    }
    return params


def convert_transform_file_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `ConvertTransformFileParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("dimensions", None) is None:
        raise StyxValidationError("`dimensions` must not be None")
    if not isinstance(params["dimensions"], int):
        raise StyxValidationError(f'`dimensions` has the wrong type: Received `{type(params.get("dimensions", None))}` expected `int`')
    if params.get("input_transform_file", None) is None:
        raise StyxValidationError("`input_transform_file` must not be None")
    if not isinstance(params["input_transform_file"], (pathlib.Path, str)):
        raise StyxValidationError(f'`input_transform_file` has the wrong type: Received `{type(params.get("input_transform_file", None))}` expected `InputPathType`')
    if params.get("output_transform_file", None) is None:
        raise StyxValidationError("`output_transform_file` must not be None")
    if not isinstance(params["output_transform_file"], str):
        raise StyxValidationError(f'`output_transform_file` has the wrong type: Received `{type(params.get("output_transform_file", None))}` expected `str`')
    if params.get("matrix", False) is None:
        raise StyxValidationError("`matrix` must not be None")
    if not isinstance(params["matrix"], bool):
        raise StyxValidationError(f'`matrix` has the wrong type: Received `{type(params.get("matrix", False))}` expected `bool`')
    if params.get("homogeneous_matrix", False) is None:
        raise StyxValidationError("`homogeneous_matrix` must not be None")
    if not isinstance(params["homogeneous_matrix"], bool):
        raise StyxValidationError(f'`homogeneous_matrix` has the wrong type: Received `{type(params.get("homogeneous_matrix", False))}` expected `bool`')
    if params.get("RAS", False) is None:
        raise StyxValidationError("`RAS` must not be None")
    if not isinstance(params["RAS"], bool):
        raise StyxValidationError(f'`RAS` has the wrong type: Received `{type(params.get("RAS", False))}` expected `bool`')
    if params.get("convert_to_affine_type", False) is None:
        raise StyxValidationError("`convert_to_affine_type` must not be None")
    if not isinstance(params["convert_to_affine_type"], bool):
        raise StyxValidationError(f'`convert_to_affine_type` has the wrong type: Received `{type(params.get("convert_to_affine_type", False))}` expected `bool`')


def convert_transform_file_cargs(
    params: ConvertTransformFileParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("ConvertTransformFile")
    cargs.append(str(params.get("dimensions", None)))
    cargs.append(execution.input_file(params.get("input_transform_file", None)))
    cargs.append(params.get("output_transform_file", None))
    if params.get("matrix", False):
        cargs.append("--matrix")
    if params.get("homogeneous_matrix", False):
        cargs.append("--homogeneousMatrix")
    if params.get("RAS", False):
        cargs.append("--RAS")
    if params.get("convert_to_affine_type", False):
        cargs.append("--convertToAffineType")
    return cargs


def convert_transform_file_outputs(
    params: ConvertTransformFileParamsDict,
    execution: Execution,
) -> ConvertTransformFileOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = ConvertTransformFileOutputs(
        root=execution.output_file("."),
    )
    return ret


def convert_transform_file_execute(
    params: ConvertTransformFileParamsDict,
    runner: Runner | None = None,
) -> ConvertTransformFileOutputs:
    """
    ConvertTransformFile
    
    Utility to read in a transform file (presumed to be in binary format) and
    output it in various formats. Default output is legacy human-readable text
    format. Without any options, the output filename extension must be .txt or
    .tfm to signify a text-formatted transform file.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `ConvertTransformFileOutputs`).
    """
    convert_transform_file_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(CONVERT_TRANSFORM_FILE_METADATA)
    params = execution.params(params)
    cargs = convert_transform_file_cargs(params, execution)
    ret = convert_transform_file_outputs(params, execution)
    execution.run(cargs)
    return ret


def convert_transform_file(
    dimensions: int,
    input_transform_file: InputPathType,
    output_transform_file: str,
    matrix: bool = False,
    homogeneous_matrix: bool = False,
    ras: bool = False,
    convert_to_affine_type: bool = False,
    runner: Runner | None = None,
) -> ConvertTransformFileOutputs:
    """
    ConvertTransformFile
    
    Utility to read in a transform file (presumed to be in binary format) and
    output it in various formats. Default output is legacy human-readable text
    format. Without any options, the output filename extension must be .txt or
    .tfm to signify a text-formatted transform file.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        dimensions: Dimensionality of the transform file.
        input_transform_file: Path to the input transform file.
        output_transform_file: Path for the output transform file.
        matrix: Output only the transform matrix to a text file, one row per\
            line with space-delimited values. Only works for specific transform\
            types.
        homogeneous_matrix: Output an N+1 square homogeneous matrix from the\
            transform matrix and offset. Only works for specific transform types.
        ras: Convert the output into the RAS coordinate system if combined with\
            'matrix' or 'homogeneousMatrix'.
        convert_to_affine_type: Convert the input transform type to\
            AffineTransform and output again as a binary transform file.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `ConvertTransformFileOutputs`).
    """
    params = convert_transform_file_params(
        dimensions=dimensions,
        input_transform_file=input_transform_file,
        output_transform_file=output_transform_file,
        matrix=matrix,
        homogeneous_matrix=homogeneous_matrix,
        ras=ras,
        convert_to_affine_type=convert_to_affine_type,
    )
    return convert_transform_file_execute(params, runner)


__all__ = [
    "CONVERT_TRANSFORM_FILE_METADATA",
    "ConvertTransformFileOutputs",
    "ConvertTransformFileParamsDict",
    "ConvertTransformFileParamsDictTagged",
    "convert_transform_file",
    "convert_transform_file_execute",
    "convert_transform_file_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

ANTS_NEUROIMAGING_BATTERY_METADATA = Metadata(
    id="3c073ef3644ad5eb7696969346c7947932d963ad.boutiques",
    name="antsNeuroimagingBattery",
    package="ants",
    container_image_tag="antsx/ants:v2.5.3",
)


_AntsNeuroimagingBatteryParamsDictNoTag = typing.TypedDict('_AntsNeuroimagingBatteryParamsDictNoTag', {
    "input_directory": str,
    "output_directory": str,
    "output_name": str,
    "anatomical_image": InputPathType,
    "anatomical_mask": InputPathType,
    "template": typing.NotRequired[InputPathType | None],
    "template_transform_name": typing.NotRequired[str | None],
    "template_labels": typing.NotRequired[InputPathType | None],
    "dti_flag": typing.NotRequired[str | None],
    "pcasl_flag": typing.NotRequired[str | None],
    "pasl_flag": typing.NotRequired[str | None],
    "pasl_m0_flag": typing.NotRequired[str | None],
    "bold_flag": typing.NotRequired[str | None],
    "rsbold_flag": typing.NotRequired[str | None],
    "mt_flag": typing.NotRequired[str | None],
    "no_mt_flag": typing.NotRequired[str | None],
    "temp_directory": typing.NotRequired[str | None],
    "help": bool,
    "info_only": bool,
})
AntsNeuroimagingBatteryParamsDictTagged = typing.TypedDict('AntsNeuroimagingBatteryParamsDictTagged', {
    "@type": typing.Literal["ants/antsNeuroimagingBattery"],
    "input_directory": str,
    "output_directory": str,
    "output_name": str,
    "anatomical_image": InputPathType,
    "anatomical_mask": InputPathType,
    "template": typing.NotRequired[InputPathType | None],
    "template_transform_name": typing.NotRequired[str | None],
    "template_labels": typing.NotRequired[InputPathType | None],
    "dti_flag": typing.NotRequired[str | None],
    "pcasl_flag": typing.NotRequired[str | None],
    "pasl_flag": typing.NotRequired[str | None],
    "pasl_m0_flag": typing.NotRequired[str | None],
    "bold_flag": typing.NotRequired[str | None],
    "rsbold_flag": typing.NotRequired[str | None],
    "mt_flag": typing.NotRequired[str | None],
    "no_mt_flag": typing.NotRequired[str | None],
    "temp_directory": typing.NotRequired[str | None],
    "help": bool,
    "info_only": bool,
})
AntsNeuroimagingBatteryParamsDict = _AntsNeuroimagingBatteryParamsDictNoTag | AntsNeuroimagingBatteryParamsDictTagged


class AntsNeuroimagingBatteryOutputs(typing.NamedTuple):
    """
    Output object returned when calling `AntsNeuroimagingBatteryParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_directory: OutputPathType
    """Directory containing output transform files."""


def ants_neuroimaging_battery_params(
    input_directory: str,
    output_directory: str,
    output_name: str,
    anatomical_image: InputPathType,
    anatomical_mask: InputPathType,
    template: InputPathType | None = None,
    template_transform_name: str | None = None,
    template_labels: InputPathType | None = None,
    dti_flag: str | None = None,
    pcasl_flag: str | None = None,
    pasl_flag: str | None = None,
    pasl_m0_flag: str | None = None,
    bold_flag: str | None = None,
    rsbold_flag: str | None = None,
    mt_flag: str | None = None,
    no_mt_flag: str | None = None,
    temp_directory: str | None = None,
    help_: bool = False,
    info_only: bool = False,
) -> AntsNeuroimagingBatteryParamsDictTagged:
    """
    Build parameters.
    
    Args:
        input_directory: Directory where to look for modality images.
        output_directory: Directory where output goes (where\
            antsCorticalThickness output lives).
        output_name: File prefix for outputs.
        anatomical_image: Reference subject image (usually T1).
        anatomical_mask: Mask of anatomical image, should contain cerebrum,\
            cerebellum, and brainstem.
        template: Template image.
        template_transform_name: Basename of transforms from anatomical to\
            template space (must be in output base dir).
        template_labels: Labels in template space.
        dti_flag: DTI flag in DIRNAME/fileflag/outid format.
        pcasl_flag: pCASL flag in DIRNAME/fileflag/outid format.
        pasl_flag: PASL flag in DIRNAME/fileflag/outid format.
        pasl_m0_flag: PASL M0 flag in DIRNAME/fileflag/outid format.
        bold_flag: BOLD flag in DIRNAME/fileflag/outid format.
        rsbold_flag: RSBOLD flag in DIRNAME/fileflag/outid format.
        mt_flag: MT flag in DIRNAME/fileflag/outid format.
        no_mt_flag: No MT flag in DIRNAME/fileflag/outid format.
        temp_directory: Temporary directory.
        help_: Display help information.
        info_only: Look for inputs, output what is there, but don't process any\
            data.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "ants/antsNeuroimagingBattery",
        "input_directory": input_directory,
        "output_directory": output_directory,
        "output_name": output_name,
        "anatomical_image": anatomical_image,
        "anatomical_mask": anatomical_mask,
        "help": help_,
        "info_only": info_only,
    }
    if template is not None:
        params["template"] = template
    if template_transform_name is not None:
        params["template_transform_name"] = template_transform_name
    if template_labels is not None:
        params["template_labels"] = template_labels
    if dti_flag is not None:
        params["dti_flag"] = dti_flag
    if pcasl_flag is not None:
        params["pcasl_flag"] = pcasl_flag
    if pasl_flag is not None:
        params["pasl_flag"] = pasl_flag
    if pasl_m0_flag is not None:
        params["pasl_m0_flag"] = pasl_m0_flag
    if bold_flag is not None:
        params["bold_flag"] = bold_flag
    if rsbold_flag is not None:
        params["rsbold_flag"] = rsbold_flag
    if mt_flag is not None:
        params["mt_flag"] = mt_flag
    if no_mt_flag is not None:
        params["no_mt_flag"] = no_mt_flag
    if temp_directory is not None:
        params["temp_directory"] = temp_directory
    return params


def ants_neuroimaging_battery_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `AntsNeuroimagingBatteryParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("input_directory", None) is None:
        raise StyxValidationError("`input_directory` must not be None")
    if not isinstance(params["input_directory"], str):
        raise StyxValidationError(f'`input_directory` has the wrong type: Received `{type(params.get("input_directory", None))}` expected `str`')
    if params.get("output_directory", None) is None:
        raise StyxValidationError("`output_directory` must not be None")
    if not isinstance(params["output_directory"], str):
        raise StyxValidationError(f'`output_directory` has the wrong type: Received `{type(params.get("output_directory", None))}` expected `str`')
    if params.get("output_name", None) is None:
        raise StyxValidationError("`output_name` must not be None")
    if not isinstance(params["output_name"], str):
        raise StyxValidationError(f'`output_name` has the wrong type: Received `{type(params.get("output_name", None))}` expected `str`')
    if params.get("anatomical_image", None) is None:
        raise StyxValidationError("`anatomical_image` must not be None")
    if not isinstance(params["anatomical_image"], (pathlib.Path, str)):
        raise StyxValidationError(f'`anatomical_image` has the wrong type: Received `{type(params.get("anatomical_image", None))}` expected `InputPathType`')
    if params.get("anatomical_mask", None) is None:
        raise StyxValidationError("`anatomical_mask` must not be None")
    if not isinstance(params["anatomical_mask"], (pathlib.Path, str)):
        raise StyxValidationError(f'`anatomical_mask` has the wrong type: Received `{type(params.get("anatomical_mask", None))}` expected `InputPathType`')
    if params.get("template", None) is not None:
        if not isinstance(params["template"], (pathlib.Path, str)):
            raise StyxValidationError(f'`template` has the wrong type: Received `{type(params.get("template", None))}` expected `InputPathType | None`')
    if params.get("template_transform_name", None) is not None:
        if not isinstance(params["template_transform_name"], str):
            raise StyxValidationError(f'`template_transform_name` has the wrong type: Received `{type(params.get("template_transform_name", None))}` expected `str | None`')
    if params.get("template_labels", None) is not None:
        if not isinstance(params["template_labels"], (pathlib.Path, str)):
            raise StyxValidationError(f'`template_labels` has the wrong type: Received `{type(params.get("template_labels", None))}` expected `InputPathType | None`')
    if params.get("dti_flag", None) is not None:
        if not isinstance(params["dti_flag"], str):
            raise StyxValidationError(f'`dti_flag` has the wrong type: Received `{type(params.get("dti_flag", None))}` expected `str | None`')
    if params.get("pcasl_flag", None) is not None:
        if not isinstance(params["pcasl_flag"], str):
            raise StyxValidationError(f'`pcasl_flag` has the wrong type: Received `{type(params.get("pcasl_flag", None))}` expected `str | None`')
    if params.get("pasl_flag", None) is not None:
        if not isinstance(params["pasl_flag"], str):
            raise StyxValidationError(f'`pasl_flag` has the wrong type: Received `{type(params.get("pasl_flag", None))}` expected `str | None`')
    if params.get("pasl_m0_flag", None) is not None:
        if not isinstance(params["pasl_m0_flag"], str):
            raise StyxValidationError(f'`pasl_m0_flag` has the wrong type: Received `{type(params.get("pasl_m0_flag", None))}` expected `str | None`')
    if params.get("bold_flag", None) is not None:
        if not isinstance(params["bold_flag"], str):
            raise StyxValidationError(f'`bold_flag` has the wrong type: Received `{type(params.get("bold_flag", None))}` expected `str | None`')
    if params.get("rsbold_flag", None) is not None:
        if not isinstance(params["rsbold_flag"], str):
            raise StyxValidationError(f'`rsbold_flag` has the wrong type: Received `{type(params.get("rsbold_flag", None))}` expected `str | None`')
    if params.get("mt_flag", None) is not None:
        if not isinstance(params["mt_flag"], str):
            raise StyxValidationError(f'`mt_flag` has the wrong type: Received `{type(params.get("mt_flag", None))}` expected `str | None`')
    if params.get("no_mt_flag", None) is not None:
        if not isinstance(params["no_mt_flag"], str):
            raise StyxValidationError(f'`no_mt_flag` has the wrong type: Received `{type(params.get("no_mt_flag", None))}` expected `str | None`')
    if params.get("temp_directory", None) is not None:
        if not isinstance(params["temp_directory"], str):
            raise StyxValidationError(f'`temp_directory` has the wrong type: Received `{type(params.get("temp_directory", None))}` expected `str | None`')
    if params.get("help", False) is None:
        raise StyxValidationError("`help` must not be None")
    if not isinstance(params["help"], bool):
        raise StyxValidationError(f'`help` has the wrong type: Received `{type(params.get("help", False))}` expected `bool`')
    if params.get("info_only", False) is None:
        raise StyxValidationError("`info_only` must not be None")
    if not isinstance(params["info_only"], bool):
        raise StyxValidationError(f'`info_only` has the wrong type: Received `{type(params.get("info_only", False))}` expected `bool`')


def ants_neuroimaging_battery_cargs(
    params: AntsNeuroimagingBatteryParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("antsNeuroimagingBattery")
    cargs.extend([
        "--input-directory",
        params.get("input_directory", None)
    ])
    cargs.extend([
        "--output-directory",
        params.get("output_directory", None)
    ])
    cargs.extend([
        "--output-name",
        params.get("output_name", None)
    ])
    cargs.extend([
        "--anatomical",
        execution.input_file(params.get("anatomical_image", None))
    ])
    cargs.extend([
        "--anatomical-mask",
        execution.input_file(params.get("anatomical_mask", None))
    ])
    if params.get("template", None) is not None:
        cargs.extend([
            "--template",
            execution.input_file(params.get("template", None))
        ])
    if params.get("template_transform_name", None) is not None:
        cargs.extend([
            "--template-transform-name",
            params.get("template_transform_name", None)
        ])
    if params.get("template_labels", None) is not None:
        cargs.extend([
            "--template-labels",
            execution.input_file(params.get("template_labels", None))
        ])
    if params.get("dti_flag", None) is not None:
        cargs.extend([
            "--dti-flag",
            params.get("dti_flag", None)
        ])
    if params.get("pcasl_flag", None) is not None:
        cargs.extend([
            "--pcasl-flag",
            params.get("pcasl_flag", None)
        ])
    if params.get("pasl_flag", None) is not None:
        cargs.extend([
            "--pasl-flag",
            params.get("pasl_flag", None)
        ])
    if params.get("pasl_m0_flag", None) is not None:
        cargs.extend([
            "--pasl-m0-flag",
            params.get("pasl_m0_flag", None)
        ])
    if params.get("bold_flag", None) is not None:
        cargs.extend([
            "--bold-flag",
            params.get("bold_flag", None)
        ])
    if params.get("rsbold_flag", None) is not None:
        cargs.extend([
            "--rsbold-flag",
            params.get("rsbold_flag", None)
        ])
    if params.get("mt_flag", None) is not None:
        cargs.extend([
            "--mt-flag",
            params.get("mt_flag", None)
        ])
    if params.get("no_mt_flag", None) is not None:
        cargs.extend([
            "--no-mt-flag",
            params.get("no_mt_flag", None)
        ])
    if params.get("temp_directory", None) is not None:
        cargs.extend([
            "--temp-directory",
            params.get("temp_directory", None)
        ])
    if params.get("help", False):
        cargs.append("--help")
    if params.get("info_only", False):
        cargs.append("--info-only")
    return cargs


def ants_neuroimaging_battery_outputs(
    params: AntsNeuroimagingBatteryParamsDict,
    execution: Execution,
) -> AntsNeuroimagingBatteryOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = AntsNeuroimagingBatteryOutputs(
        root=execution.output_file("."),
        output_directory=execution.output_file(params.get("output_directory", None)),
    )
    return ret


def ants_neuroimaging_battery_execute(
    params: AntsNeuroimagingBatteryParamsDict,
    runner: Runner | None = None,
) -> AntsNeuroimagingBatteryOutputs:
    """
    antsNeuroimagingBattery
    
    Align MR modalities to a common within-subject (and optional template)
    space.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `AntsNeuroimagingBatteryOutputs`).
    """
    ants_neuroimaging_battery_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(ANTS_NEUROIMAGING_BATTERY_METADATA)
    params = execution.params(params)
    cargs = ants_neuroimaging_battery_cargs(params, execution)
    ret = ants_neuroimaging_battery_outputs(params, execution)
    execution.run(cargs)
    return ret


def ants_neuroimaging_battery(
    input_directory: str,
    output_directory: str,
    output_name: str,
    anatomical_image: InputPathType,
    anatomical_mask: InputPathType,
    template: InputPathType | None = None,
    template_transform_name: str | None = None,
    template_labels: InputPathType | None = None,
    dti_flag: str | None = None,
    pcasl_flag: str | None = None,
    pasl_flag: str | None = None,
    pasl_m0_flag: str | None = None,
    bold_flag: str | None = None,
    rsbold_flag: str | None = None,
    mt_flag: str | None = None,
    no_mt_flag: str | None = None,
    temp_directory: str | None = None,
    help_: bool = False,
    info_only: bool = False,
    runner: Runner | None = None,
) -> AntsNeuroimagingBatteryOutputs:
    """
    antsNeuroimagingBattery
    
    Align MR modalities to a common within-subject (and optional template)
    space.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        input_directory: Directory where to look for modality images.
        output_directory: Directory where output goes (where\
            antsCorticalThickness output lives).
        output_name: File prefix for outputs.
        anatomical_image: Reference subject image (usually T1).
        anatomical_mask: Mask of anatomical image, should contain cerebrum,\
            cerebellum, and brainstem.
        template: Template image.
        template_transform_name: Basename of transforms from anatomical to\
            template space (must be in output base dir).
        template_labels: Labels in template space.
        dti_flag: DTI flag in DIRNAME/fileflag/outid format.
        pcasl_flag: pCASL flag in DIRNAME/fileflag/outid format.
        pasl_flag: PASL flag in DIRNAME/fileflag/outid format.
        pasl_m0_flag: PASL M0 flag in DIRNAME/fileflag/outid format.
        bold_flag: BOLD flag in DIRNAME/fileflag/outid format.
        rsbold_flag: RSBOLD flag in DIRNAME/fileflag/outid format.
        mt_flag: MT flag in DIRNAME/fileflag/outid format.
        no_mt_flag: No MT flag in DIRNAME/fileflag/outid format.
        temp_directory: Temporary directory.
        help_: Display help information.
        info_only: Look for inputs, output what is there, but don't process any\
            data.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `AntsNeuroimagingBatteryOutputs`).
    """
    params = ants_neuroimaging_battery_params(
        input_directory=input_directory,
        output_directory=output_directory,
        output_name=output_name,
        anatomical_image=anatomical_image,
        anatomical_mask=anatomical_mask,
        template=template,
        template_transform_name=template_transform_name,
        template_labels=template_labels,
        dti_flag=dti_flag,
        pcasl_flag=pcasl_flag,
        pasl_flag=pasl_flag,
        pasl_m0_flag=pasl_m0_flag,
        bold_flag=bold_flag,
        rsbold_flag=rsbold_flag,
        mt_flag=mt_flag,
        no_mt_flag=no_mt_flag,
        temp_directory=temp_directory,
        help_=help_,
        info_only=info_only,
    )
    return ants_neuroimaging_battery_execute(params, runner)


__all__ = [
    "ANTS_NEUROIMAGING_BATTERY_METADATA",
    "AntsNeuroimagingBatteryOutputs",
    "AntsNeuroimagingBatteryParamsDict",
    "AntsNeuroimagingBatteryParamsDictTagged",
    "ants_neuroimaging_battery",
    "ants_neuroimaging_battery_execute",
    "ants_neuroimaging_battery_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

ANTS_BRAIN_EXTRACTION_SH_METADATA = Metadata(
    id="72830e1ccd1043926fd7dcd995dc980ded440138.boutiques",
    name="antsBrainExtraction.sh",
    package="ants",
    container_image_tag="antsx/ants:v2.5.3",
)


_AntsBrainExtractionShParamsDictNoTag = typing.TypedDict('_AntsBrainExtractionShParamsDictNoTag', {
    "image_dimension": typing.Literal[2, 3],
    "anatomical_image": InputPathType,
    "template": InputPathType,
    "probability_mask": InputPathType,
    "tissue_classification": typing.NotRequired[str | None],
    "brain_extraction_registration_mask": typing.NotRequired[InputPathType | None],
    "keep_temporary_files": typing.NotRequired[bool | None],
    "single_floating_point_precision": typing.NotRequired[bool | None],
    "initial_moving_transform": typing.NotRequired[InputPathType | None],
    "rotation_search_params": typing.NotRequired[str | None],
    "image_file_suffix": typing.NotRequired[str | None],
    "translation_search_params": typing.NotRequired[str | None],
    "random_seeding": typing.NotRequired[bool | None],
    "debug_mode": typing.NotRequired[int | None],
    "output_prefix": typing.NotRequired[str | None],
})
AntsBrainExtractionShParamsDictTagged = typing.TypedDict('AntsBrainExtractionShParamsDictTagged', {
    "@type": typing.Literal["ants/antsBrainExtraction.sh"],
    "image_dimension": typing.Literal[2, 3],
    "anatomical_image": InputPathType,
    "template": InputPathType,
    "probability_mask": InputPathType,
    "tissue_classification": typing.NotRequired[str | None],
    "brain_extraction_registration_mask": typing.NotRequired[InputPathType | None],
    "keep_temporary_files": typing.NotRequired[bool | None],
    "single_floating_point_precision": typing.NotRequired[bool | None],
    "initial_moving_transform": typing.NotRequired[InputPathType | None],
    "rotation_search_params": typing.NotRequired[str | None],
    "image_file_suffix": typing.NotRequired[str | None],
    "translation_search_params": typing.NotRequired[str | None],
    "random_seeding": typing.NotRequired[bool | None],
    "debug_mode": typing.NotRequired[int | None],
    "output_prefix": typing.NotRequired[str | None],
})
AntsBrainExtractionShParamsDict = _AntsBrainExtractionShParamsDictNoTag | AntsBrainExtractionShParamsDictTagged


class AntsBrainExtractionShOutputs(typing.NamedTuple):
    """
    Output object returned when calling `AntsBrainExtractionShParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    brain_extracted_image: OutputPathType | None
    """Brain extracted image"""
    brain_mask: OutputPathType | None
    """Brain extraction mask"""
    brain_segmentation: OutputPathType | None
    """Brain segmentation (tissue classification)"""
    csf_segmentation: OutputPathType | None
    """Cerebrospinal fluid segmentation"""
    gm_segmentation: OutputPathType | None
    """Gray matter segmentation"""
    wm_segmentation: OutputPathType | None
    """White matter segmentation"""
    generic_affine: OutputPathType | None
    """Generic affine transform from registration"""
    warp_field: OutputPathType | None
    """Warp field from registration"""
    inverse_warp_field: OutputPathType | None
    """Inverse warp field from registration"""


def ants_brain_extraction_sh_params(
    image_dimension: typing.Literal[2, 3],
    anatomical_image: InputPathType,
    template: InputPathType,
    probability_mask: InputPathType,
    tissue_classification: str | None = None,
    brain_extraction_registration_mask: InputPathType | None = None,
    keep_temporary_files: bool | None = None,
    single_floating_point_precision: bool | None = None,
    initial_moving_transform: InputPathType | None = None,
    rotation_search_params: str | None = None,
    image_file_suffix: str | None = None,
    translation_search_params: str | None = None,
    random_seeding: bool | None = None,
    debug_mode: int | None = None,
    output_prefix: str | None = None,
) -> AntsBrainExtractionShParamsDictTagged:
    """
    Build parameters.
    
    Args:
        image_dimension: Image dimension (2 or 3 for 2- or 3-dimensional image).
        anatomical_image: Anatomical image (Structural image, typically T1). If\
            more than one anatomical image is specified, subsequently specified\
            images are used during the segmentation process. However, only the\
            first image is used in the registration of priors.
        template: Brain extraction template (Anatomical template).
        probability_mask: Brain extraction probability mask (with intensity\
            range 1=definitely brain to 0=definitely background).
        tissue_classification: Tissue classification - A k-means segmentation\
            to find tissue classes. Format: KxcsfLabelxgmLabelxwmLabel. Examples:\
            '3x1x2x3' for T1 (K=3, CSF=1, GM=2, WM=3), '3x3x2x1' for T2, '3x1x3x2'\
            for FLAIR, '4x4x2x3' for K=4.
        brain_extraction_registration_mask: Brain extraction registration mask\
            (Mask used for registration to limit the metric computation to a\
            specific region).
        keep_temporary_files: Keep temporary files (keep brain\
            extraction/segmentation warps, etc). 0=delete, 1=keep.
        single_floating_point_precision: Use antsRegistration with single (1)\
            or double (0) floating point precision.
        initial_moving_transform: Initial moving transform (An ITK affine\
            transform, e.g., from antsAI or ITK-SNAP). Without this option, the\
            script calls antsAI to search for a good initial moving transform.
        rotation_search_params: Rotation search parameters for antsAI in format\
            'step,arcFraction'. Step is in degrees, arcFraction goes from 0 (no\
            search) to 1 (search -180 to 180 degrees).
        image_file_suffix: Image file suffix (any standard ITK IO format, e.g.,\
            nrrd, nii.gz, mhd).
        translation_search_params: Translation search parameters for antsAI in\
            format 'step,range'. Step is in mm, -range to range will be tested in\
            each dimension.
        random_seeding: Use random seeding: 1=use random seed from system\
            clock, 0=use fixed seed (ANTS_RANDOM_SEED=19650218 or value from\
            environment). For reproducibility, set to 0 and also set\
            ITK_GLOBAL_DEFAULT_NUMBER_OF_THREADS=1.
        debug_mode: Test/debug mode: If > 0, runs a faster version of the\
            script. Only for debugging, results will not be good.
        output_prefix: Output prefix (directory + file prefix).
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "ants/antsBrainExtraction.sh",
        "image_dimension": image_dimension,
        "anatomical_image": anatomical_image,
        "template": template,
        "probability_mask": probability_mask,
    }
    if tissue_classification is not None:
        params["tissue_classification"] = tissue_classification
    if brain_extraction_registration_mask is not None:
        params["brain_extraction_registration_mask"] = brain_extraction_registration_mask
    if keep_temporary_files is not None:
        params["keep_temporary_files"] = keep_temporary_files
    if single_floating_point_precision is not None:
        params["single_floating_point_precision"] = single_floating_point_precision
    if initial_moving_transform is not None:
        params["initial_moving_transform"] = initial_moving_transform
    if rotation_search_params is not None:
        params["rotation_search_params"] = rotation_search_params
    if image_file_suffix is not None:
        params["image_file_suffix"] = image_file_suffix
    if translation_search_params is not None:
        params["translation_search_params"] = translation_search_params
    if random_seeding is not None:
        params["random_seeding"] = random_seeding
    if debug_mode is not None:
        params["debug_mode"] = debug_mode
    if output_prefix is not None:
        params["output_prefix"] = output_prefix
    return params


def ants_brain_extraction_sh_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `AntsBrainExtractionShParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("image_dimension", None) is None:
        raise StyxValidationError("`image_dimension` must not be None")
    if not isinstance(params["image_dimension"], int):
        raise StyxValidationError(f'`image_dimension` has the wrong type: Received `{type(params.get("image_dimension", None))}` expected `typing.Literal[2, 3]`')
    if params["image_dimension"] not in [2, 3]:
        raise StyxValidationError("Parameter `image_dimension` must be one of [2, 3]")
    if params.get("anatomical_image", None) is None:
        raise StyxValidationError("`anatomical_image` must not be None")
    if not isinstance(params["anatomical_image"], (pathlib.Path, str)):
        raise StyxValidationError(f'`anatomical_image` has the wrong type: Received `{type(params.get("anatomical_image", None))}` expected `InputPathType`')
    if params.get("template", None) is None:
        raise StyxValidationError("`template` must not be None")
    if not isinstance(params["template"], (pathlib.Path, str)):
        raise StyxValidationError(f'`template` has the wrong type: Received `{type(params.get("template", None))}` expected `InputPathType`')
    if params.get("probability_mask", None) is None:
        raise StyxValidationError("`probability_mask` must not be None")
    if not isinstance(params["probability_mask"], (pathlib.Path, str)):
        raise StyxValidationError(f'`probability_mask` has the wrong type: Received `{type(params.get("probability_mask", None))}` expected `InputPathType`')
    if params.get("tissue_classification", None) is not None:
        if not isinstance(params["tissue_classification"], str):
            raise StyxValidationError(f'`tissue_classification` has the wrong type: Received `{type(params.get("tissue_classification", None))}` expected `str | None`')
    if params.get("brain_extraction_registration_mask", None) is not None:
        if not isinstance(params["brain_extraction_registration_mask"], (pathlib.Path, str)):
            raise StyxValidationError(f'`brain_extraction_registration_mask` has the wrong type: Received `{type(params.get("brain_extraction_registration_mask", None))}` expected `InputPathType | None`')
    if params.get("keep_temporary_files", None) is not None:
        if not isinstance(params["keep_temporary_files"], bool):
            raise StyxValidationError(f'`keep_temporary_files` has the wrong type: Received `{type(params.get("keep_temporary_files", None))}` expected `bool | None`')
    if params.get("single_floating_point_precision", None) is not None:
        if not isinstance(params["single_floating_point_precision"], bool):
            raise StyxValidationError(f'`single_floating_point_precision` has the wrong type: Received `{type(params.get("single_floating_point_precision", None))}` expected `bool | None`')
    if params.get("initial_moving_transform", None) is not None:
        if not isinstance(params["initial_moving_transform"], (pathlib.Path, str)):
            raise StyxValidationError(f'`initial_moving_transform` has the wrong type: Received `{type(params.get("initial_moving_transform", None))}` expected `InputPathType | None`')
    if params.get("rotation_search_params", None) is not None:
        if not isinstance(params["rotation_search_params"], str):
            raise StyxValidationError(f'`rotation_search_params` has the wrong type: Received `{type(params.get("rotation_search_params", None))}` expected `str | None`')
    if params.get("image_file_suffix", None) is not None:
        if not isinstance(params["image_file_suffix"], str):
            raise StyxValidationError(f'`image_file_suffix` has the wrong type: Received `{type(params.get("image_file_suffix", None))}` expected `str | None`')
    if params.get("translation_search_params", None) is not None:
        if not isinstance(params["translation_search_params"], str):
            raise StyxValidationError(f'`translation_search_params` has the wrong type: Received `{type(params.get("translation_search_params", None))}` expected `str | None`')
    if params.get("random_seeding", None) is not None:
        if not isinstance(params["random_seeding"], bool):
            raise StyxValidationError(f'`random_seeding` has the wrong type: Received `{type(params.get("random_seeding", None))}` expected `bool | None`')
    if params.get("debug_mode", None) is not None:
        if not isinstance(params["debug_mode"], int):
            raise StyxValidationError(f'`debug_mode` has the wrong type: Received `{type(params.get("debug_mode", None))}` expected `int | None`')
        if params["debug_mode"] < 0:
            raise StyxValidationError("Parameter `debug_mode` must be at least 0")
    if params.get("output_prefix", None) is not None:
        if not isinstance(params["output_prefix"], str):
            raise StyxValidationError(f'`output_prefix` has the wrong type: Received `{type(params.get("output_prefix", None))}` expected `str | None`')


def ants_brain_extraction_sh_cargs(
    params: AntsBrainExtractionShParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("antsBrainExtraction.sh")
    cargs.extend([
        "-d",
        str(params.get("image_dimension", None))
    ])
    cargs.extend([
        "-a",
        execution.input_file(params.get("anatomical_image", None))
    ])
    cargs.extend([
        "-e",
        execution.input_file(params.get("template", None))
    ])
    cargs.extend([
        "-m",
        execution.input_file(params.get("probability_mask", None))
    ])
    if params.get("tissue_classification", None) is not None:
        cargs.extend([
            "-c",
            params.get("tissue_classification", None)
        ])
    if params.get("brain_extraction_registration_mask", None) is not None:
        cargs.extend([
            "-f",
            execution.input_file(params.get("brain_extraction_registration_mask", None))
        ])
    if params.get("keep_temporary_files", None) is not None:
        cargs.extend([
            "-k",
            ("1" if params.get("keep_temporary_files", None) else "0")
        ])
    if params.get("single_floating_point_precision", None) is not None:
        cargs.extend([
            "-q",
            ("1" if params.get("single_floating_point_precision", None) else "0")
        ])
    if params.get("initial_moving_transform", None) is not None:
        cargs.extend([
            "-r",
            execution.input_file(params.get("initial_moving_transform", None))
        ])
    if params.get("rotation_search_params", None) is not None:
        cargs.extend([
            "-R",
            params.get("rotation_search_params", None)
        ])
    if params.get("image_file_suffix", None) is not None:
        cargs.extend([
            "-s",
            params.get("image_file_suffix", None)
        ])
    if params.get("translation_search_params", None) is not None:
        cargs.extend([
            "-T",
            params.get("translation_search_params", None)
        ])
    if params.get("random_seeding", None) is not None:
        cargs.extend([
            "-u",
            ("1" if params.get("random_seeding", None) else "0")
        ])
    if params.get("debug_mode", None) is not None:
        cargs.extend([
            "-z",
            str(params.get("debug_mode", None))
        ])
    if params.get("output_prefix", None) is not None:
        cargs.extend([
            "-o",
            params.get("output_prefix", None)
        ])
    return cargs


def ants_brain_extraction_sh_outputs(
    params: AntsBrainExtractionShParamsDict,
    execution: Execution,
) -> AntsBrainExtractionShOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = AntsBrainExtractionShOutputs(
        root=execution.output_file("."),
        brain_extracted_image=execution.output_file(params.get("output_prefix", None) + "BrainExtractionBrain.nii.gz") if (params.get("output_prefix") is not None) else None,
        brain_mask=execution.output_file(params.get("output_prefix", None) + "BrainExtractionMask.nii.gz") if (params.get("output_prefix") is not None) else None,
        brain_segmentation=execution.output_file(params.get("output_prefix", None) + "BrainExtractionSegmentation.nii.gz") if (params.get("output_prefix") is not None) else None,
        csf_segmentation=execution.output_file(params.get("output_prefix", None) + "BrainExtractionCSF.nii.gz") if (params.get("output_prefix") is not None) else None,
        gm_segmentation=execution.output_file(params.get("output_prefix", None) + "BrainExtractionGM.nii.gz") if (params.get("output_prefix") is not None) else None,
        wm_segmentation=execution.output_file(params.get("output_prefix", None) + "BrainExtractionWM.nii.gz") if (params.get("output_prefix") is not None) else None,
        generic_affine=execution.output_file(params.get("output_prefix", None) + "BrainExtractionPrior0GenericAffine.mat") if (params.get("output_prefix") is not None) else None,
        warp_field=execution.output_file(params.get("output_prefix", None) + "BrainExtractionPrior1Warp.nii.gz") if (params.get("output_prefix") is not None) else None,
        inverse_warp_field=execution.output_file(params.get("output_prefix", None) + "BrainExtractionPrior1InverseWarp.nii.gz") if (params.get("output_prefix") is not None) else None,
    )
    return ret


def ants_brain_extraction_sh_execute(
    params: AntsBrainExtractionShParamsDict,
    runner: Runner | None = None,
) -> AntsBrainExtractionShOutputs:
    """
    antsBrainExtraction.sh
    
    antsBrainExtraction.sh performs template-based brain extraction using N4
    bias correction, registration, and Atropos segmentation.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `AntsBrainExtractionShOutputs`).
    """
    ants_brain_extraction_sh_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(ANTS_BRAIN_EXTRACTION_SH_METADATA)
    params = execution.params(params)
    cargs = ants_brain_extraction_sh_cargs(params, execution)
    ret = ants_brain_extraction_sh_outputs(params, execution)
    execution.run(cargs)
    return ret


def ants_brain_extraction_sh(
    image_dimension: typing.Literal[2, 3],
    anatomical_image: InputPathType,
    template: InputPathType,
    probability_mask: InputPathType,
    tissue_classification: str | None = None,
    brain_extraction_registration_mask: InputPathType | None = None,
    keep_temporary_files: bool | None = None,
    single_floating_point_precision: bool | None = None,
    initial_moving_transform: InputPathType | None = None,
    rotation_search_params: str | None = None,
    image_file_suffix: str | None = None,
    translation_search_params: str | None = None,
    random_seeding: bool | None = None,
    debug_mode: int | None = None,
    output_prefix: str | None = None,
    runner: Runner | None = None,
) -> AntsBrainExtractionShOutputs:
    """
    antsBrainExtraction.sh
    
    antsBrainExtraction.sh performs template-based brain extraction using N4
    bias correction, registration, and Atropos segmentation.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        image_dimension: Image dimension (2 or 3 for 2- or 3-dimensional image).
        anatomical_image: Anatomical image (Structural image, typically T1). If\
            more than one anatomical image is specified, subsequently specified\
            images are used during the segmentation process. However, only the\
            first image is used in the registration of priors.
        template: Brain extraction template (Anatomical template).
        probability_mask: Brain extraction probability mask (with intensity\
            range 1=definitely brain to 0=definitely background).
        tissue_classification: Tissue classification - A k-means segmentation\
            to find tissue classes. Format: KxcsfLabelxgmLabelxwmLabel. Examples:\
            '3x1x2x3' for T1 (K=3, CSF=1, GM=2, WM=3), '3x3x2x1' for T2, '3x1x3x2'\
            for FLAIR, '4x4x2x3' for K=4.
        brain_extraction_registration_mask: Brain extraction registration mask\
            (Mask used for registration to limit the metric computation to a\
            specific region).
        keep_temporary_files: Keep temporary files (keep brain\
            extraction/segmentation warps, etc). 0=delete, 1=keep.
        single_floating_point_precision: Use antsRegistration with single (1)\
            or double (0) floating point precision.
        initial_moving_transform: Initial moving transform (An ITK affine\
            transform, e.g., from antsAI or ITK-SNAP). Without this option, the\
            script calls antsAI to search for a good initial moving transform.
        rotation_search_params: Rotation search parameters for antsAI in format\
            'step,arcFraction'. Step is in degrees, arcFraction goes from 0 (no\
            search) to 1 (search -180 to 180 degrees).
        image_file_suffix: Image file suffix (any standard ITK IO format, e.g.,\
            nrrd, nii.gz, mhd).
        translation_search_params: Translation search parameters for antsAI in\
            format 'step,range'. Step is in mm, -range to range will be tested in\
            each dimension.
        random_seeding: Use random seeding: 1=use random seed from system\
            clock, 0=use fixed seed (ANTS_RANDOM_SEED=19650218 or value from\
            environment). For reproducibility, set to 0 and also set\
            ITK_GLOBAL_DEFAULT_NUMBER_OF_THREADS=1.
        debug_mode: Test/debug mode: If > 0, runs a faster version of the\
            script. Only for debugging, results will not be good.
        output_prefix: Output prefix (directory + file prefix).
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `AntsBrainExtractionShOutputs`).
    """
    params = ants_brain_extraction_sh_params(
        image_dimension=image_dimension,
        anatomical_image=anatomical_image,
        template=template,
        probability_mask=probability_mask,
        tissue_classification=tissue_classification,
        brain_extraction_registration_mask=brain_extraction_registration_mask,
        keep_temporary_files=keep_temporary_files,
        single_floating_point_precision=single_floating_point_precision,
        initial_moving_transform=initial_moving_transform,
        rotation_search_params=rotation_search_params,
        image_file_suffix=image_file_suffix,
        translation_search_params=translation_search_params,
        random_seeding=random_seeding,
        debug_mode=debug_mode,
        output_prefix=output_prefix,
    )
    return ants_brain_extraction_sh_execute(params, runner)


__all__ = [
    "ANTS_BRAIN_EXTRACTION_SH_METADATA",
    "AntsBrainExtractionShOutputs",
    "AntsBrainExtractionShParamsDict",
    "AntsBrainExtractionShParamsDictTagged",
    "ants_brain_extraction_sh",
    "ants_brain_extraction_sh_execute",
    "ants_brain_extraction_sh_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

ANTS_INTRODUCTION_SH_METADATA = Metadata(
    id="2466e97f47783a3d8aa1ce52756260c6e54db2c7.boutiques",
    name="antsIntroduction.sh",
    package="ants",
    container_image_tag="antsx/ants:v2.5.3",
)


_AntsIntroductionShParamsDictNoTag = typing.TypedDict('_AntsIntroductionShParamsDictNoTag', {
    "image_dimension": typing.Literal[2, 3],
    "reference_image": InputPathType,
    "input_image": InputPathType,
    "force": typing.NotRequired[bool | None],
    "labels_in_fixed_image_space": typing.NotRequired[str | None],
    "max_iterations": typing.NotRequired[int | None],
    "n4_bias_field_correction": typing.NotRequired[bool | None],
    "outprefix": typing.NotRequired[str | None],
    "quality_check": typing.NotRequired[bool | None],
    "similarity_metric": typing.NotRequired[str | None],
    "transformation_model": typing.NotRequired[str | None],
})
AntsIntroductionShParamsDictTagged = typing.TypedDict('AntsIntroductionShParamsDictTagged', {
    "@type": typing.Literal["ants/antsIntroduction.sh"],
    "image_dimension": typing.Literal[2, 3],
    "reference_image": InputPathType,
    "input_image": InputPathType,
    "force": typing.NotRequired[bool | None],
    "labels_in_fixed_image_space": typing.NotRequired[str | None],
    "max_iterations": typing.NotRequired[int | None],
    "n4_bias_field_correction": typing.NotRequired[bool | None],
    "outprefix": typing.NotRequired[str | None],
    "quality_check": typing.NotRequired[bool | None],
    "similarity_metric": typing.NotRequired[str | None],
    "transformation_model": typing.NotRequired[str | None],
})
AntsIntroductionShParamsDict = _AntsIntroductionShParamsDictNoTag | AntsIntroductionShParamsDictTagged


class AntsIntroductionShOutputs(typing.NamedTuple):
    """
    Output object returned when calling `AntsIntroductionShParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def ants_introduction_sh_params(
    image_dimension: typing.Literal[2, 3],
    reference_image: InputPathType,
    input_image: InputPathType,
    force: bool | None = None,
    labels_in_fixed_image_space: str | None = None,
    max_iterations: int | None = None,
    n4_bias_field_correction: bool | None = None,
    outprefix: str | None = None,
    quality_check: bool | None = None,
    similarity_metric: str | None = None,
    transformation_model: str | None = None,
) -> AntsIntroductionShParamsDictTagged:
    """
    Build parameters.
    
    Args:
        image_dimension: Image dimension for registration: 2 or 3.
        reference_image: Reference image for registration.
        input_image: Input image to be registered.
        force: Force script to proceed even if headers may be incompatible.
        labels_in_fixed_image_space: Labels in fixed image space to deform to\
            moving image.
        max_iterations: Maximum number of iterations.
        n4_bias_field_correction: N4 Bias Field Correction of moving image: 0\
            for off, 1 for on.
        outprefix: A prefix that is prepended to all output files.
        quality_check: Perform a Quality Check (QC) of the result: 0 for off, 1\
            for on.
        similarity_metric: Type of similarity metric used for registration.
        transformation_model: Type of transformation model used for\
            registration.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "ants/antsIntroduction.sh",
        "image_dimension": image_dimension,
        "reference_image": reference_image,
        "input_image": input_image,
    }
    if force is not None:
        params["force"] = force
    if labels_in_fixed_image_space is not None:
        params["labels_in_fixed_image_space"] = labels_in_fixed_image_space
    if max_iterations is not None:
        params["max_iterations"] = max_iterations
    if n4_bias_field_correction is not None:
        params["n4_bias_field_correction"] = n4_bias_field_correction
    if outprefix is not None:
        params["outprefix"] = outprefix
    if quality_check is not None:
        params["quality_check"] = quality_check
    if similarity_metric is not None:
        params["similarity_metric"] = similarity_metric
    if transformation_model is not None:
        params["transformation_model"] = transformation_model
    return params


def ants_introduction_sh_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `AntsIntroductionShParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("image_dimension", None) is None:
        raise StyxValidationError("`image_dimension` must not be None")
    if not isinstance(params["image_dimension"], int):
        raise StyxValidationError(f'`image_dimension` has the wrong type: Received `{type(params.get("image_dimension", None))}` expected `typing.Literal[2, 3]`')
    if params["image_dimension"] not in [2, 3]:
        raise StyxValidationError("Parameter `image_dimension` must be one of [2, 3]")
    if params.get("reference_image", None) is None:
        raise StyxValidationError("`reference_image` must not be None")
    if not isinstance(params["reference_image"], (pathlib.Path, str)):
        raise StyxValidationError(f'`reference_image` has the wrong type: Received `{type(params.get("reference_image", None))}` expected `InputPathType`')
    if params.get("input_image", None) is None:
        raise StyxValidationError("`input_image` must not be None")
    if not isinstance(params["input_image"], (pathlib.Path, str)):
        raise StyxValidationError(f'`input_image` has the wrong type: Received `{type(params.get("input_image", None))}` expected `InputPathType`')
    if params.get("force", None) is not None:
        if not isinstance(params["force"], bool):
            raise StyxValidationError(f'`force` has the wrong type: Received `{type(params.get("force", None))}` expected `bool | None`')
    if params.get("labels_in_fixed_image_space", None) is not None:
        if not isinstance(params["labels_in_fixed_image_space"], str):
            raise StyxValidationError(f'`labels_in_fixed_image_space` has the wrong type: Received `{type(params.get("labels_in_fixed_image_space", None))}` expected `str | None`')
    if params.get("max_iterations", None) is not None:
        if not isinstance(params["max_iterations"], int):
            raise StyxValidationError(f'`max_iterations` has the wrong type: Received `{type(params.get("max_iterations", None))}` expected `int | None`')
    if params.get("n4_bias_field_correction", None) is not None:
        if not isinstance(params["n4_bias_field_correction"], bool):
            raise StyxValidationError(f'`n4_bias_field_correction` has the wrong type: Received `{type(params.get("n4_bias_field_correction", None))}` expected `bool | None`')
    if params.get("outprefix", None) is not None:
        if not isinstance(params["outprefix"], str):
            raise StyxValidationError(f'`outprefix` has the wrong type: Received `{type(params.get("outprefix", None))}` expected `str | None`')
    if params.get("quality_check", None) is not None:
        if not isinstance(params["quality_check"], bool):
            raise StyxValidationError(f'`quality_check` has the wrong type: Received `{type(params.get("quality_check", None))}` expected `bool | None`')
    if params.get("similarity_metric", None) is not None:
        if not isinstance(params["similarity_metric"], str):
            raise StyxValidationError(f'`similarity_metric` has the wrong type: Received `{type(params.get("similarity_metric", None))}` expected `str | None`')
    if params.get("transformation_model", None) is not None:
        if not isinstance(params["transformation_model"], str):
            raise StyxValidationError(f'`transformation_model` has the wrong type: Received `{type(params.get("transformation_model", None))}` expected `str | None`')


def ants_introduction_sh_cargs(
    params: AntsIntroductionShParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("antsIntroduction.sh")
    cargs.extend([
        "-d",
        str(params.get("image_dimension", None))
    ])
    cargs.extend([
        "-r",
        execution.input_file(params.get("reference_image", None))
    ])
    cargs.extend([
        "-i",
        execution.input_file(params.get("input_image", None))
    ])
    if params.get("force", None) is not None:
        cargs.extend([
            "-f",
            ("1" if params.get("force", None) else "0")
        ])
    if params.get("labels_in_fixed_image_space", None) is not None:
        cargs.extend([
            "-l",
            params.get("labels_in_fixed_image_space", None)
        ])
    if params.get("max_iterations", None) is not None:
        cargs.extend([
            "-m",
            str(params.get("max_iterations", None))
        ])
    if params.get("n4_bias_field_correction", None) is not None:
        cargs.extend([
            "-n",
            ("1" if params.get("n4_bias_field_correction", None) else "0")
        ])
    if params.get("outprefix", None) is not None:
        cargs.extend([
            "-o",
            params.get("outprefix", None)
        ])
    if params.get("quality_check", None) is not None:
        cargs.extend([
            "-q",
            ("1" if params.get("quality_check", None) else "0")
        ])
    if params.get("similarity_metric", None) is not None:
        cargs.extend([
            "-s",
            params.get("similarity_metric", None)
        ])
    if params.get("transformation_model", None) is not None:
        cargs.extend([
            "-t",
            params.get("transformation_model", None)
        ])
    return cargs


def ants_introduction_sh_outputs(
    params: AntsIntroductionShParamsDict,
    execution: Execution,
) -> AntsIntroductionShOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = AntsIntroductionShOutputs(
        root=execution.output_file("."),
    )
    return ret


def ants_introduction_sh_execute(
    params: AntsIntroductionShParamsDict,
    runner: Runner | None = None,
) -> AntsIntroductionShOutputs:
    """
    antsIntroduction.sh
    
    Script for registration using ANTS with compulsory and optional arguments.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `AntsIntroductionShOutputs`).
    """
    ants_introduction_sh_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(ANTS_INTRODUCTION_SH_METADATA)
    params = execution.params(params)
    cargs = ants_introduction_sh_cargs(params, execution)
    ret = ants_introduction_sh_outputs(params, execution)
    execution.run(cargs)
    return ret


def ants_introduction_sh(
    image_dimension: typing.Literal[2, 3],
    reference_image: InputPathType,
    input_image: InputPathType,
    force: bool | None = None,
    labels_in_fixed_image_space: str | None = None,
    max_iterations: int | None = None,
    n4_bias_field_correction: bool | None = None,
    outprefix: str | None = None,
    quality_check: bool | None = None,
    similarity_metric: str | None = None,
    transformation_model: str | None = None,
    runner: Runner | None = None,
) -> AntsIntroductionShOutputs:
    """
    antsIntroduction.sh
    
    Script for registration using ANTS with compulsory and optional arguments.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        image_dimension: Image dimension for registration: 2 or 3.
        reference_image: Reference image for registration.
        input_image: Input image to be registered.
        force: Force script to proceed even if headers may be incompatible.
        labels_in_fixed_image_space: Labels in fixed image space to deform to\
            moving image.
        max_iterations: Maximum number of iterations.
        n4_bias_field_correction: N4 Bias Field Correction of moving image: 0\
            for off, 1 for on.
        outprefix: A prefix that is prepended to all output files.
        quality_check: Perform a Quality Check (QC) of the result: 0 for off, 1\
            for on.
        similarity_metric: Type of similarity metric used for registration.
        transformation_model: Type of transformation model used for\
            registration.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `AntsIntroductionShOutputs`).
    """
    params = ants_introduction_sh_params(
        image_dimension=image_dimension,
        reference_image=reference_image,
        input_image=input_image,
        force=force,
        labels_in_fixed_image_space=labels_in_fixed_image_space,
        max_iterations=max_iterations,
        n4_bias_field_correction=n4_bias_field_correction,
        outprefix=outprefix,
        quality_check=quality_check,
        similarity_metric=similarity_metric,
        transformation_model=transformation_model,
    )
    return ants_introduction_sh_execute(params, runner)


__all__ = [
    "ANTS_INTRODUCTION_SH_METADATA",
    "AntsIntroductionShOutputs",
    "AntsIntroductionShParamsDict",
    "AntsIntroductionShParamsDictTagged",
    "ants_introduction_sh",
    "ants_introduction_sh_execute",
    "ants_introduction_sh_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

SMOOTH_IMAGE_METADATA = Metadata(
    id="0b9f787af4c1a0fcf3db27f547e0fd544c29f18f.boutiques",
    name="SmoothImage",
    package="ants",
    container_image_tag="antsx/ants:v2.5.3",
)


_SmoothImageParamsDictNoTag = typing.TypedDict('_SmoothImageParamsDictNoTag', {
    "image_dimension": int,
    "image_ext": InputPathType,
    "smoothing_sigma": str,
    "out_image_ext": str,
    "sigma_units": typing.NotRequired[bool | None],
    "median_filter": typing.NotRequired[bool | None],
})
SmoothImageParamsDictTagged = typing.TypedDict('SmoothImageParamsDictTagged', {
    "@type": typing.Literal["ants/SmoothImage"],
    "image_dimension": int,
    "image_ext": InputPathType,
    "smoothing_sigma": str,
    "out_image_ext": str,
    "sigma_units": typing.NotRequired[bool | None],
    "median_filter": typing.NotRequired[bool | None],
})
SmoothImageParamsDict = _SmoothImageParamsDictNoTag | SmoothImageParamsDictTagged


class SmoothImageOutputs(typing.NamedTuple):
    """
    Output object returned when calling `SmoothImageParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    smoothed_image: OutputPathType
    """The output smoothed image file."""


def smooth_image_params(
    image_dimension: int,
    image_ext: InputPathType,
    smoothing_sigma: str,
    out_image_ext: str,
    sigma_units: bool | None = None,
    median_filter: bool | None = None,
) -> SmoothImageParamsDictTagged:
    """
    Build parameters.
    
    Args:
        image_dimension: Specifies the dimensionality of the image.
        image_ext: The input image file to be smoothed.
        smoothing_sigma: The sigma value for smoothing. A separate sigma may be\
            specified for each dimension, e.g., '1.5x1x2'.
        out_image_ext: The output smoothed image file.
        sigma_units: Determines if sigma is in spacing units (1) or not (0).\
            Default is 0.
        median_filter: Whether to use median filter. Default is 0. If using\
            median filter, sigma represents the radius in voxels.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "ants/SmoothImage",
        "image_dimension": image_dimension,
        "image_ext": image_ext,
        "smoothing_sigma": smoothing_sigma,
        "out_image_ext": out_image_ext,
    }
    if sigma_units is not None:
        params["sigma_units"] = sigma_units
    if median_filter is not None:
        params["median_filter"] = median_filter
    return params


def smooth_image_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `SmoothImageParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("image_dimension", None) is None:
        raise StyxValidationError("`image_dimension` must not be None")
    if not isinstance(params["image_dimension"], int):
        raise StyxValidationError(f'`image_dimension` has the wrong type: Received `{type(params.get("image_dimension", None))}` expected `int`')
    if params.get("image_ext", None) is None:
        raise StyxValidationError("`image_ext` must not be None")
    if not isinstance(params["image_ext"], (pathlib.Path, str)):
        raise StyxValidationError(f'`image_ext` has the wrong type: Received `{type(params.get("image_ext", None))}` expected `InputPathType`')
    if params.get("smoothing_sigma", None) is None:
        raise StyxValidationError("`smoothing_sigma` must not be None")
    if not isinstance(params["smoothing_sigma"], str):
        raise StyxValidationError(f'`smoothing_sigma` has the wrong type: Received `{type(params.get("smoothing_sigma", None))}` expected `str`')
    if params.get("out_image_ext", None) is None:
        raise StyxValidationError("`out_image_ext` must not be None")
    if not isinstance(params["out_image_ext"], str):
        raise StyxValidationError(f'`out_image_ext` has the wrong type: Received `{type(params.get("out_image_ext", None))}` expected `str`')
    if params.get("sigma_units", None) is not None:
        if not isinstance(params["sigma_units"], bool):
            raise StyxValidationError(f'`sigma_units` has the wrong type: Received `{type(params.get("sigma_units", None))}` expected `bool | None`')
    if params.get("median_filter", None) is not None:
        if not isinstance(params["median_filter"], bool):
            raise StyxValidationError(f'`median_filter` has the wrong type: Received `{type(params.get("median_filter", None))}` expected `bool | None`')


def smooth_image_cargs(
    params: SmoothImageParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("SmoothImage")
    cargs.append(str(params.get("image_dimension", None)))
    cargs.append(execution.input_file(params.get("image_ext", None)))
    cargs.append(params.get("smoothing_sigma", None))
    cargs.append(params.get("out_image_ext", None))
    if params.get("sigma_units", None) is not None:
        cargs.append(("1" if params.get("sigma_units", None) else "0"))
    if params.get("median_filter", None) is not None:
        cargs.append(("1" if params.get("median_filter", None) else "0"))
    return cargs


def smooth_image_outputs(
    params: SmoothImageParamsDict,
    execution: Execution,
) -> SmoothImageOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = SmoothImageOutputs(
        root=execution.output_file("."),
        smoothed_image=execution.output_file(params.get("out_image_ext", None)),
    )
    return ret


def smooth_image_execute(
    params: SmoothImageParamsDict,
    runner: Runner | None = None,
) -> SmoothImageOutputs:
    """
    SmoothImage
    
    SmoothImage allows smoothing of images with adjustable sigma values,
    offering optional median filtering functionality.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `SmoothImageOutputs`).
    """
    smooth_image_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(SMOOTH_IMAGE_METADATA)
    params = execution.params(params)
    cargs = smooth_image_cargs(params, execution)
    ret = smooth_image_outputs(params, execution)
    execution.run(cargs)
    return ret


def smooth_image(
    image_dimension: int,
    image_ext: InputPathType,
    smoothing_sigma: str,
    out_image_ext: str,
    sigma_units: bool | None = None,
    median_filter: bool | None = None,
    runner: Runner | None = None,
) -> SmoothImageOutputs:
    """
    SmoothImage
    
    SmoothImage allows smoothing of images with adjustable sigma values,
    offering optional median filtering functionality.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        image_dimension: Specifies the dimensionality of the image.
        image_ext: The input image file to be smoothed.
        smoothing_sigma: The sigma value for smoothing. A separate sigma may be\
            specified for each dimension, e.g., '1.5x1x2'.
        out_image_ext: The output smoothed image file.
        sigma_units: Determines if sigma is in spacing units (1) or not (0).\
            Default is 0.
        median_filter: Whether to use median filter. Default is 0. If using\
            median filter, sigma represents the radius in voxels.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `SmoothImageOutputs`).
    """
    params = smooth_image_params(
        image_dimension=image_dimension,
        image_ext=image_ext,
        smoothing_sigma=smoothing_sigma,
        out_image_ext=out_image_ext,
        sigma_units=sigma_units,
        median_filter=median_filter,
    )
    return smooth_image_execute(params, runner)


__all__ = [
    "SMOOTH_IMAGE_METADATA",
    "SmoothImageOutputs",
    "SmoothImageParamsDict",
    "SmoothImageParamsDictTagged",
    "smooth_image",
    "smooth_image_execute",
    "smooth_image_params",
]
